#' Calculate KL-Divergence
#'
#' \code{KLDdist} computes the KLDistance, returning a nominal relative entropy value if two distribution vectors are supplied, or a distance matrix if one data frame is supplied.
#'
#' @param dist1 May either be a distribution vector, or a dataframe/matrix/2D array wherein each row represents a distribution vector
#' @param dist2 Optional input. Vector of counts/frequencies for a distribtuion
#' @param isFreq Optional logical argument. Default is TRUE, indicating dat has been supplied as pre-normalized frequency vectors. User may provide data in the form of counts distributions if isFreq is FALSE.
#' @param logbase Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).
#' @param returnSymetric Optional logical argument. Default is TRUE. KL Divergence is not an intrinsically symetric estimator. If TRUE, KL Divergence will be made symmetric by calculating relative entropy using either observed distribution vector as the reference (q) distributions and adding the results. If FALSE, the asymetric relative entropy value will be returned
#' @param runfast Optional logical argument. Default if FALSE. If TRUE, data matrix will be passed to Dist utility in the Rdast package to speed up computation of large distance matrices. With this option, only a symmetric dissimlarity metric is returned, log base cannot be modified, and Dirichlets Correction cannot be utilized.
#' @param DirichletCorrection Optional logical argument. Default is FALSE. Only availiable when isFreq is FALSE. Should a Dirichlets correction be applied to frequency vectors before relative entropy is calculated. Allows for entropy to be calculated if q may contain 0 due to under-sampling.
#'
#' @details KL-Divergence calculates the relative entropy between two discrete distributions. The two distributions may be provided either as frequency (normalized to sum to 1) or as count vectors, but must have the same domain of support (ie - be the same length)
#'
#' If two vectors are supplied, a nominal entropy value is returned, as in the relativeEntropy.
#'
#' Alternatively a dataframe/matrix/2D array may be provided wherein each row each row represents a distribution vector, for which a dissimilarity matrix is returned representing each combination of pairwise comparisons between rows. If returnSymetric if FALSE, the dissimilarity will be assymetric, as each value in the matrix will represent the relative entropy estimate for which the column index serves as the reference distribution. If returnSymetric is TRUE, the the dissimilarity matrix will be made symetric by summming over the diagonal.
#'
#' It should be noted that if KLD Distance is being calculated for under-sampled data that may contain zero-counts, relative entropy will be returned as Inf. If data is provided as counts, a Dirchelets correction may be applied to prevent these missing values to facilitate downstream analyses. In general, however, KLD Distance may not be an appropriate dissimilarity statistic for such data, so use this option with due caution.
#'
#' @examples
#'
#' set.seed(10)
#' dist1 <- matrix(rmultinom(5, 100, c(0.2, 0.2, 0.2, 0.2, 0.2)), nrow = 5, ncol = 5)
#'
#' KLDdist(dist1, isFreq = F, returnSymetric = TRUE)
#' KLDdist(dist1, isFreq = F, returnSymetric = FALSE)
#'
#' dist1[1,1] <- 0
#' KLDdist(dist1, isFreq = F, returnSymetric = TRUE)
#' KLDdist(dist1, isFreq = F, returnSymetric = FALSE)
#'
#' KLDdist(dist1, isFreq = F, returnSymetric = TRUE, DirichletCorrection = TRUE)
#' KLDdist(dist1, isFreq = F, runfast = T)
#'
#' @export


KLDdist <- function(dist1, dist2 = NULL, isFreq = TRUE, logbase = 2, returnSymetric = TRUE, runfast = FALSE, DirichletCorrection = FALSE){

  if(!is.null(dist2)){ #distance betwee two vectors

    if(isFreq & DirichletCorrection){
      stop('Error: Dirichlet correction only availiable for count data. See relativeEntropy function for more options')
    }

    if(returnSymetric){
      return( relativeEntropy(dist1, dist2, isFreq = isFreq, logbase = logbase, DirichletCorrection = DirichletCorrection) + relativeEntropy(dist2, dist1, isFreq = isFreq, logbase = logbase, DirichletCorrection = DirichletCorrection) )
    }else{
      return( relativeEntropy(dist1, dist2, isFreq = isFreq, logbase = logbase, DirichletCorrection = DirichletCorrection) )
    }

  }else{ # compute distance matrix for rows (obs) of data matrix

    if(sum(is.na(dist1)) > 0){
      stop('Error: Data contains missing NA values')
    }

    if(!isFreq){

      if(runfast){

        # normalize counts
        N <- apply(dist1, 1, sum)
        dist1 <- t(apply(dist1, 1, function(x) x/sum(x)))

        # compute distance matrix
        dout <- Rfast::Dist(dist1, method = 'kullback_leibler') #NOTE: Only returns symmetric KL dist, and can't set the log base
        rownames(dout) <- rownames(dist1)
        colnames(dout) <- rownames(dist1)

      }else{

        dout <- apply(dist1, 1, function(x_i) sapply(1:nrow(dist1), function(j) KLDdist(x_i, dist1[j, ], isFreq = F, logbase = logbase, DirichletCorrection = DirichletCorrection)))

      }
    }else{

      if( sum(round(apply(dist1, 1, sum), 5) != 1) > 0 ){ # check normalization
        #print(apply(dist1, 1, sum)[apply(dist1, 1, sum) != 1])
        stop('Error: All rows must sum to 1 to be a legitimate probability distribution')
      }

      if(runfast){
        dout <- Rfast::Dist(dist1, method = 'kullback_leibler') #NOTE: Only returns symmetric KL dist, and can't set the log base
        rownames(dout) <- rownames(dist1)
        colnames(dout) <- rownames(dist1)
      }else{
        if(DirichletCorrection){
          stop('Error: Dirichlet Correction only availiable for count data')
        }
        dout <- apply(dist1, 1, function(x_i) sapply(1:nrow(dist1), function(j) KLDdist(x_i, dist1[j, ], isFreq = T, logbase = logbase)))
      }
    }

    if(!returnSymetric & !runfast){
      return(dout)
    }else if(returnSymetric & !runfast){
      return(dout + t(dout))
    }else{
      return(as.dist(dout))
    }

  }

}
