#' Mutual Information Test
#'
#' \code{MITest} evaluates if a statistically significant bivariate relationship exists between two discretely encoded variables via a permtuation test that uses mutual information estimator.
#'
#' @param codex First encoded variable. User may pass either a list object from LIT clustering output (for a single cluster number) or a data frame formatted with named rows and one column containing the encoded variable.
#' @param codey Second encoded variable. User may pass either a list object from LIT clustering output (for a single cluster number) or a data frame formatted with named rows and one column containing the encoded variable.
#' @param nperm Optional argument. Defualt is 1000. Number of permutations used to simulate the null MI value for each pairwise comparision of candidate tree encodings
#' @param logbase Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).
#' @param estimator Optional argument. Method employed to estimate empirical frequencies of the joint encoding used to calculate entropic values. Defualt is 'ML'. Bias-corrected estimators available include "shrink", "Jeffreys","Laplace", "SG", "minmax".
#' @param rescale Optional logical argument. If TRUE (default), estimator used in permutation tests for signicance of association is MI rescaled by the total entropy of the joint encoding. If FALSE, the raw/unscaled MI estimate is used as the standard of comparison.
#' @param plotresults Optional logical argument. Default is FALSE. If TRUE returns a histogram plot of MI values under the null compared to the observed value.
#' @param verbose Optional logical argument. Default is FALSE. If TRUE additional simulation specs are returned.
#'
#' @details MItest compares the observed Mututal information of the observed joint distribution, and compares it to the distribution of MI values calculated under the null using a random permutation of indices to break any bivariate relationships between axes.
#'
#' @return A named list containing:
#' \item{PVals}{Integer value. Proportion of MI values simulated under the null that are equal to or larger than the observed value}
#' \item{ObsEntropy}{Mutual Information estimate of the observed joint encoding}
#' \item{testSpecs}{Named list of MI test options used to generate simulation results}
#' \item{ObsEntropy}{Returned only if verbose is TRUE. Named list containing complete output of information theoretic estimates (from calcEntropy) for the observed joint encoding}
#' \item{NullDis}{Returned only if verbose is TRUE. Vector containing MI values simulated under the null}
#' \item{Plot}{Returned only if verbose is TRUE. Null if plotresults is FALSE. Visualization of distribution of MI values simulated under the null}
#'
#' @examples
#'
#' myx <- round(runif(100, 0, 5))
#' myy <- round(runif(100, 0, 5))
#'
#' MITest(myx, myy, plotresults = T, verbose = T)
#'
#'
#' @export

MITest <- function(codex, codey,
                   nperm = 1000, logbase = 2,
                   estimator = 'ML',
                   rescale = T,
                   verbose = F,
                   plotresults = F){
  # codex = data vector of row categorical variable
  # codey = dat vector of column categorical variable
  # nperm = number of permutations
  # NOTE: x and y should by aligned by observed individuals

  if(is.data.frame(codex)){
    if(1 %in% dim(codex)){
      temp <- codex[,1]
      names(temp) <- rownames(codex)
      codex <- temp
    }else{
      stop('Error: Encoding X must be data vector')
    }
  }

  if(is.data.frame(codey)){
    if(1 %in% dim(codey)){
      temp <- codey[,1]
      names(temp) <- rownames(codey)
      codey <- temp
    }else{
      stop('Error: Encoding Y must be data vector')
    }
  }

  if( sum(names(codey) != names(codex)) > 0 ){
    stop('Error: Observations do not align between input vectors')
  }

  obsinformtation <- calcEntropy(table(codex,codey),
                                 logbase = logbase,
                                 estimator = estimator)
  if(rescale){
    entobs <- obsinformtation$RelMI
  }else{
    entobs <- obsinformtation$MutualInformation
  }


  randent <- rep(NA, nperm)
  for(i in 1:nperm){
    xr <- sample(codex, replace = F)
    yr <- sample(codey, replace = F)

    if(rescale){
      randent[i] <- calcEntropy(table(xr, yr),
                                logbase = logbase,
                                estimator = estimator)$RelMI
    }else{
      randent[i] <- calcEntropy(table(xr, yr),
                                logbase = logbase,
                                estimator = estimator)$MutualInformation
    }
  }

  plotout <- NULL
  if(plotresults){
    xmin <- min(c(randent,entobs))-0.1*abs(min(c(randent,entobs)))
    xmax <- max(c(randent,entobs))+0.1*abs(max(c(randent,entobs)))
    plotout <- ggplot2::qplot(randent,
                     main = 'Histogram of Randomized Mutual Information Values',
                     xlim = c(xmin,xmax),
                     bins = 10) +
      ggplot2::geom_vline(xintercept=entobs, col = 'red')
    print(plotout)
  }

  if(verbose){
    return(list(pval = sum(randent>=entobs)/nperm,
                ObsCounts = table(codex,codey),
                ObsEntropy = obsinformtation,
                NullDist = randent,
                Plot = plotout,
                testSpecs = list(
                  logbase = logbase,
                  nperm = nperm,
                  rescale = rescale,
                  estimator = estimator
                )
    ))
  }else{
    return(list(pval = sum(randent>=entobs)/nperm,
                ObsEntropy = obsinformtation,
                testSpecs = list(
                  logbase = logbase,
                  nperm = nperm,
                  rescale = rescale,
                  estimator = estimator
                )
    ))
  }

}
