#' Mutual Information Test for Tree Structures
#'
#' \code{bivarTreeTest} allows users to test for a significant bivariate relationship between two discretized variables using an MI test on a grid of encoding granularities (encoding metaparameter).
#'
#' @param inputdatx First encoded variable. User may pass either a list output from other LIT clustering utilities, a dendrogram, or a data frame formatted with named rows and one column containing the encoded variable.
#' @param inputdaty Second encoded variable. User may pass either a list output from other LIT clustering utilities, a dendrogram, or a data frame formatted with named rows and one column containing the encoded variable.
#' @param grid_x Vectors of integer values containing the range of potential cluster numbers with which to encode variable x.
#' @param grid_y Vectors of integer values containing the range of potential cluster numbers with which to encode variable y.
#' @param logbase Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).
#' @param nperm Optional argument. Defualt is 1000. Number of permutations used to simulate the null MI value for each pairwise comparision of candidate tree encodings
#' @param estimator Optional argument. Method employed to estimate empirical frequencies of the joint encoding used to calculate entropic values. Defualt is 'ML'. Bias-corrected estimators available include "shrink", "Jeffreys","Laplace", "SG", "minmax".
#' @param rescaleMI Optional logical argument. If TRUE (default), estimator used in permutation tests for signicance of association is MI rescaled by the total entropy of the joint encoding. If FALSE, the raw/unscaled MI estimate is used as the standard of comparison.
#' @param showplot Optional logical argument. Default is TRUE, which returns a heatmap visualization of the best MI test results for each metaprameter combination across the grid of cluster number values.
#'
#' @details When working with complex data sets that are empirically discretized using clustering techniques, there is typically no obvious or intrisic granularity in the subseqent encoding. Subsequently, it can be challenging to determine a priori the optimal combination of encoding granularities to test for a significant bivariate relationship between encodings.
#'
#' The bivarTreeTest utility allows users to conduct mutual information tests on a grid of encoding granularity values, and provides visualization of simulation results to aid in selecting the optimal metaparameter values where bivariate relationship come into optimal focus. This ultimately provides a more comprehsive assessment of the relationships between tree structures.
#'
#' Users may input either LIT clustering output, raw dendrograms, or a discrete encoding provided as a dataframe. If the user passes output from a data mechanics clustering (dmplot), multiple MI tests will be conducted at each encoding granularity gridpoint, with full results returned in SimResults, and the best test result used in the optimization visualization. This allows users to not only explore the best encoding granularity between observational units, but even optimize for temporal/multivariate structures across experimental units.
#'
#' @return A structured list containing:
#' \item{PVals}{A multidimensional array, with indices matching the clustering indices, containing the p-value results for the corresponding MI test}
#' \item{MIVals}{A multidimensional array, with indices matching the clustering indices, containing the observed MI values of the encoded data}
#' \item{ZScores}{A multidimensional array,  with indices matching the clustering indices, containing the observed MI values of the encoded data centered and scaled by the median and inter-quartile range of the MI values simulated under the null}
#' \item{PlotOut}{NULL if showplot is FALSE. A pheatmap object containing a visualization of the best simulated MI Z-scores across the metaparmeter grid of clustering granularities.}
#'
#' @examples
#'
#' library(labstats)
#' data(locomotor)
#'
#' locowide <- unstack(locomotor, dist~time)
#' rownames(locowide) <- paste('Rat', unique(locomotor$animal), sep = '')
#' names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
#' auxrow <- unique(locomotor[, c('animal', 'drug')])
#' rownames(auxrow) <- paste('Rat', auxrow$animal, sep = '')
#' auxrow <- auxrow[,'drug', drop = F]
#' names(auxrow) <- 'Treatment'
#'
#' encodeout <- encodePlot(locowide, dist(locowide), dist(t(locowide)),
#'                     n_rowclusters = 2:5, n_colclusters = 0,
#'                     plot_title = 'Rat Locomotion Pattern Encoding',
#'                     annotation_row = auxrow)
#'
#' myx <- as.data.frame(x = round(runif(dim(locowide)[1], 0, 5)))
#' rownames(myx) <- rownames(auxrow)
#'
#' bvout <- bivarTreeTest(encodeout, myx, grid_x = 2:5, grid_y = 5)
#' bvout$PVals
#'
#' @export






bivarTreeTest <- function(inputdatx, inputdaty,
                            grid_x = 2:5, grid_y = 2:5,
                            logbase = 2, nperm = 1000,
                            estimator = 'ML', rescaleMI = T,
                            showplot = T){


  if(1 %in% grid_x | 1 %in% grid_y ) {
    stop("Error: Cluster size must be >1")
  }


  ### Pre-process vector with fixed encoding

  if(is.data.frame(inputdatx)){

    if(.row_names_info(inputdatx)<0){stop('Error: Data frame must have rownames corresponding to observation ID')}
    if(dim(inputdatx)[2] > 1){stop('Error: If providing encoding vector, must be formated with ncol(inputdatx) = 1 ')}

    tempdat <- inputdatx[,1]
    names(tempdat) <- rownames(inputdatx)
    rownames_x <- rownames(inputdatx)

    grid_x <- sum(!is.na(unique(inputdatx)))

    temp <- list()
    temp[[paste('nrow', grid_x, 'ncol_0', sep='_')]] <-
      list(rowclust = tempdat)
    inputdatx <- temp

    dimx <- "ncol_0"


  }


  if(is.data.frame(inputdaty)){

    if(.row_names_info(inputdaty)<0){stop('Error: Data frame must have rownames corresponding to observation ID')}
    if(dim(inputdaty)[2] > 1){stop('Error: If providing encoding vector, must be formated with ncol(inputdaty) = 1 ')}

    tempdat <- inputdaty[,1]
    names(tempdat) <- rownames(inputdaty)
    rownames_y <- rownames(inputdaty)

    grid_y <- sum(!is.na(unique(inputdaty)))

    temp <- list()
    temp[[paste('nrow', grid_y, 'ncol_0', sep='_')]] <-
      list(rowclust = tempdat)
    inputdaty <- temp

    dimy <- "ncol_0"

  }

  ### Pre-process tree inputs ###

  if(dendextend::is.hclust(inputdatx)){ # format x tree as list

    rownames_x <- inputdatx$labels

    temp <- list()
    for(i in grid_x){
      temp[[paste('nrow',i,'ncol_0',sep='_')]] <- list(rowclust = cutree(inputdatx,i))
    }
    inputdatx <- temp

    dimx <- "ncol_0"
  }

  if(dendextend::is.hclust(inputdaty)){ # format y tree as list

    rownames_y <- inputdaty$labels

    temp <- list()
    for(i in grid_y){
      temp[[paste('nrow',i,'ncol_0',sep='_')]] <- list(rowclust = cutree(inputdaty,i))
    }
    inputdaty <- temp

    dimy <- "ncol_0"
  }


  ### Pre-process list inputs ###

  if(is.list(inputdatx)){

    temp <- names(inputdatx)[grep(paste('nrow_', grid_x[1], sep = ''), names(inputdatx))]
    dimx <- sub(paste('nrow_', grid_x[1], '_', sep = ''), '', temp)

    rownames_x <- names(inputdatx[[temp[1]]]$rowclust)


  }

  if(is.list(inputdaty)){

    temp <- names(inputdaty)[grep(paste('nrow_', grid_y[1], sep = ''), names(inputdaty))]
    dimy <- sub(paste('nrow_', grid_y[1], '_', sep = ''), '', temp)

    rownames_y <- names(inputdaty[[temp[1]]]$rowclust)

  }



  ## initialize testing grid ##

  simout <- array(0,
                  dim = c(length(grid_x), length(grid_y),
                          length(dimx), length(dimy),
                          nperm),
                  dimnames = list(paste('Kx=', grid_x, sep=''),
                                  paste('Ky=', grid_y, sep=''),
                                  paste('XClust_', dimx, sep=''),
                                  paste('YClust_', dimy, sep=''),
                                  1:nperm)
  )
  p_out <- simout

  mi_out <- array(0,
                  dim = c(length(grid_x), length(grid_y),
                          length(dimx), length(dimy) ),
                  dimnames = list(paste('Kx=', grid_x, sep=''),
                                  paste('Ky=', grid_y, sep=''),
                                  dimx, dimy)
  )




  ## conduct sim ##

  for(b in 1:nperm){

    # permute indices of tree (to be conserved across grid)

    permx <- data.frame(NewID = rownames_x, row.names = rownames_x)
    permx$NewID <- sample(rownames_x)

    permy <- data.frame(NewID = rownames_y, row.names = rownames_y)
    permy$NewID <- sample(rownames_y)



    for(i in 1:length(grid_x)){
      for(j in 1:length(grid_y)){

        index_x <- grep(paste('nrow_', grid_x[i], sep = ''),
                        names(inputdatx))
        index_y <- grep(paste('nrow_', grid_y[j], sep = ''),
                        names(inputdaty))

        # pull encoding values for dat x
        if(length(index_x) == 1){

          # pull encoding data frame from list
          datx <- as.data.frame(inputdatx[[index_x]]$rowclust)
          colnames(datx) <- names(inputdatx)[index_x]
          datx_obs <- datx

          # permute using the conserved tree permutation index
          tempdat <- merge(datx, permx, by = 0)
          tempdat$Row.names <- tempdat$NewID
          datx <- tempdat[ , -which(names(tempdat) == 'NewID') ]
          datx <- transform(datx,
                            row.names=Row.names,
                            Row.names=NULL)

        }else{

          # initialize data frame
          datx <- as.data.frame(inputdatx[[index_x[1]]]$rowclust)
          colnames(datx) <- names(inputdatx)[index_x[1]]

          # merge additional columns
          for(ix in 2:length(index_x)){
            temp <- as.data.frame(inputdatx[[index_x[ix]]]$rowclust)
            colnames(temp) <- names(inputdatx)[index_x[ix]]

            datx <- transform(merge(datx, temp, by = 0, all = T),
                              row.names=Row.names, Row.names=NULL)
          }
          datx_obs <- datx

          # permute using the conserved tree permutation index
          tempdat <- merge(datx, permx, by = 0)
          tempdat$Row.names <- tempdat$NewID
          datx <- tempdat[ , -which(names(tempdat) == 'NewID') ]
          datx <- transform(datx,
                            row.names=Row.names,
                            Row.names=NULL)

        }


        # pull encoding values for dat y
        if(length(index_y) == 1){

          daty <- as.data.frame(inputdaty[[index_y]]$rowclust)
          colnames(daty) <- names(inputdaty)[index_y]
          daty_obs <- daty

          # permute using the conserved tree permutation index
          tempdat <- merge(daty, permy, by = 0)
          tempdat$Row.names <- tempdat$NewID
          daty <- tempdat[ , -which(names(tempdat) == 'NewID') ]
          daty <- transform(daty,
                            row.names=Row.names,
                            Row.names=NULL)

        }else{

          # initialize data frame
          daty <- as.data.frame(inputdaty[[index_y[1]]]$rowclust)
          colnames(daty) <- names(inputdaty)[index_y[1]]

          # merge additional columns
          for(iy in 2:length(index_y)){
            temp <- as.data.frame(inputdaty[[index_y[iy]]]$rowclust)
            colnames(temp) <- names(inputdaty)[index_y[iy]]

            daty <- transform(merge(daty, temp, by = 0, all = T),
                              row.names=Row.names, Row.names=NULL)
          }
          daty_obs <- daty

          # permute using the conserved tree permutation index
          tempdat <- merge(daty, permy, by = 0)
          tempdat$Row.names <- tempdat$NewID
          daty <- tempdat[ , -which(names(tempdat) == 'NewID') ]
          daty <- transform(daty,
                            row.names=Row.names,
                            Row.names=NULL)

        }


        datall <- transform(merge(datx, daty, by = 0),
                            row.names=Row.names, Row.names=NULL)
        names(datall) <- c(paste('XCluster', names(datx)),
                           paste('YCluster',names(daty)))

        if(b == 1){
          datall_obs <- transform(merge(datx_obs, daty_obs, by = 0),
                                  row.names=Row.names, Row.names=NULL)
          names(datall_obs) <- c(paste('XCluster', names(datx_obs)),
                                 paste('YCluster',names(daty_obs)))
        }

        for(ci in 1:ncol(datx)){
          for(cj in (1:ncol(daty)) + ncol(datx) ){

            if(b == 1){
              entoutobs <- calcEntropy(table(datall_obs[,ci],
                                             datall_obs[,cj]),
                                       logbase = logbase,
                                       estimator = estimator)
              if(rescaleMI){
                mi_out[i,j,ci,cj-ncol(datx)] <- entoutobs$RelMI
              }else{
                mi_out[i,j,ci,cj-ncol(datx)] <- entoutobs$MutualInformation
              }
            }


            entout <- calcEntropy(table(datall[,ci],
                                        datall[,cj]),
                                  logbase = logbase,
                                  estimator = estimator)

            if(rescaleMI){
              simout[i,j,ci,cj-ncol(datx),b] <- entout$RelMI
            }else{
              simout[i,j,ci,cj-ncol(datx),b] <- entout$MutualInformation
            }


            p_out[i,j,ci,cj-ncol(datx),b] <- simout[i,j,ci,cj-ncol(datx),b] >= mi_out[i,j,ci,cj-ncol(datx)]

          }
        }

      }# for j
    }# for i
  }# for b

  # hist(simout[1,1,1,1,])
  # abline(v = mi_out[1,1,1,1], col = 'red')
  # sum(simout[1,1,1,1,] >= mi_out[1,1,1,1]) / length(simout[1,1,1,1,])



  ## Calculate Probabilities ##

  p_outp <- apply(p_out, c(1,2, 3, 4),
                  function(x) sum(x)/dim(p_out)[5] )

  nullmed <- apply(simout, c(1,2,3,4), median)
  nullIQR <- apply(simout, c(1,2,3,4), IQR)

  z_out <- (mi_out - nullmed)/nullIQR

  plotout <- NULL
  if(showplot){

    annocol = NULL
    annorow = NULL
    mycolist = list()
    if(dim(mi_out)[3] > 1 | dim(mi_out)[4] > 1){

      temp2 <- array(NA,
                     dim = c(dim(z_out)[1], dim(z_out)[2], 2),
                     dimnames = list(dimnames(z_out)[[1]],
                                     dimnames(z_out)[[2]],
                                     c('BestX','BestY')))
      for(t_kx in 1:dim(z_out)[1]){
        for(t_ky in 1:dim(z_out)[2]){

          temp <- z_out[t_kx, t_ky, , , drop = F]
          myindeces <- which(temp == max(temp), arr.ind = T)
          temp2[t_kx, t_ky, 'BestX']<-dimnames(z_out)[[3]][myindeces[3]]
          temp2[t_kx, t_ky, 'BestY']<-dimnames(z_out)[[4]][myindeces[4]]

        }
      }

      if(dim(mi_out)[3] > 1){

        if(dim(mi_out)[2] == 1){
          annorow <- as.data.frame(t(temp2[,,1]))
          names(annorow) <- dimnames(temp2)[[2]]
        }else{
          annorow <- as.data.frame(temp2[,,1])
        }


        for(i in 1:ncol(annorow)){
          annorow[,i] <- factor(annorow[,i],
                                levels = dimnames(mi_out)[[3]],
                                ordered = T)
          mycolist[as.character(names(annorow)[i])] <- list(setNames(brewer.pal(dim(mi_out)[3], "Greens"),dimnames(mi_out)[[3]]))
        }
      }

      if(dim(mi_out)[4] > 1){

        if(dim(mi_out)[1] == 1){
          annocol <- as.data.frame(temp2[,,2])
          names(annocol) <- dimnames(temp2)[[1]]
        }else{
          annocol <- as.data.frame(t(temp2[,,2]))
        }


        for(i in 1:ncol(annocol)){
          annocol[,i] <- factor(annocol[,i],
                                levels = dimnames(mi_out)[[4]],
                                ordered = T)
          mycolist[as.character(names(annocol)[i])] <- list(setNames(brewer.pal(dim(mi_out)[4], "Purples"),dimnames(mi_out)[[4]]))
        }
        #str(annocol)
      }

    }


    plotout <- pheatmap::pheatmap(apply(z_out, c(1,2), max),
                                  cluster_rows = F, cluster_cols = F,
                                  main = 'Simulated ZScore Estimates of Observed Relative Entropy Estimates',
                                  annotation_col = annocol,
                                  annotation_row = annorow,
                                  annotation_colors = mycolist)
    plotout
  }



  return(list(PVals = p_outp,
              MIVals = mi_out,
              ZScores = z_out,
              PlotOut = plotout))

}
