#' Calculate Entropic Measures
#'
#' \code{calcEntropy} computes an array of information theoretic measures to characterize univariate, bivariate, or multivariate systems of discrete encodings
#'
#' @param counts Vector, matrix, or multidimensional array containing count data, wherein each axis represents a discete encoding.
#' @param logbase Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).
#' @param estimator Optional argument. Estimator used to calculate empirical fequencies. Defualt is 'ML'. Bias-corrected estimators available include "shrink", "Jeffreys","Laplace", "SG", "minmax".
#' @param isfreq Optional arguement. Default is FALSE, which assumes input is a count matrix. When TRUE you may directly pass a freqeuncy matrix into entropy claculations, with no normalization or bias correction.
#' @param verbose Optional arguement. Default is FALSE. If TRUE, output will contain frequency estimates used to calculate entropic values.
#' @param ... Additional options passed to entropy() calculated using the entropy package is an estimator of type not 'ML' is specified.
#'
#' @details calcEntropy returns a range of useful information theoretic estimates for univ-, bi-, and tri-variate encodings.
#'
#' By default, entorpy is calulated using the Maximum Likelihood Estimator. Where the number of observations in the dataset is not considerably larger than the number of categories that a discrete variable is divided into (N >> k), univariate entropy estimates will be negatively biased, which will in turn bias all higher-order mutual information estimates. This will be especially true for use cases where the marginal or joint probability distributions contain many low-probability bins with observation counts of zero. Where this is an issue, the user may choose from several bias-corrected empirical frequency estimation techniques computed using the Entropy package (Hausser & Strimmer 2021).
#'
#' @return Returns a list containing:
#'
#' For univariate data:
#' \item{Entropy}{Estimate of stochasiticity of observed ensemble}
#' \item{MaxEntropy}{Estimate of the maximum theoretical stochasticity that could be acheived in an observed system if all encodings were equi-probable}
#' \item{Redundancy}{Comparison of the stochasiticty of an observed ensemble to the maximum theoretically-achievable entropy of that system}
#'
#' For bivariate data:
#' \item{MarginalEntropyRow}{List containing univariate entropy values of observations for the row variable encoding aggregated across the column margin}
#' \item{MarginalEntropyCol}{List containing univariate entropy values of observations for the column variable encoding aggregated across the row margin}
#' \item{JointEntropy}{Entropy of the joint encoding}
#' \item{ConditionEntropyRow}{Conditional entropy remaining in the row variable after accounting for the information contained in the column variable encoding}
#' \item{ConditionEntropyCol}{Conditional entropy remaining in the column variable after accounting for the information contained in the row variable encoding}
#' \item{MutualInformation}{Mutual information shared between encodings of the row and column variables}
#' \item{RelativeMutualInformation}{Mutual information rescaled by the joint entropy. Proportion of total joint entropy shared between the two encodings}
#'
#' For trivariate data:
#'
#' \item{JointEntropy}{Entropy of the joint encoding}
#' \item{MutualInformation}{Mutual information shared between encodings of the row and column variables. Positive values indicate informational redundancy, while negative values may indicate synergy between variables (see James & Crutchfield 2016). It should be noted that in sparsely sampled datasets, empirical frequency estimates tend to produce negative bias in three-way mutual information estimates}
#' \item{MCE_XY}{Mutual information shared between encodings X and Y but not with encoding Z}
#' \item{MCE_XZ}{Mutual information shared between encodings X and Z but not with encoding Y}
#' \item{MCE_YZ}{Mutual information shared between encodings Y and Z but not with encoding X}
#' \item{MarginalMI_XY}{List containing the marginal bivariate entropic estimates for encodings X and Y independent of encoding Z}
#' \item{MarginalMI_XZ}{List containing the marginal bivariate entropic estimates for encodings X and Z independent of encoding Y}
#' \item{MarginalMI_YZ}{List containing the marginal bivariate entropic estimates for encodings Y and Z independent of encoding X}
#' \item{TotalCorrelation}{Sum of three-way mutual information and all three mutual conditional entropy estimates}
#' \item{MI_TotCor}{Three-way mutual information scaled/normalized by the total correlation estimate}
#'
#'
#' @details Formulae for information theoretic estimates implemented in this function follow conventions from Mackay (2003)
#'
#' @examples
#'
#' # Example 1: Univariate data
#'
#' obsdata <- c('A', 'B', 'C', 'C') % observed data from system that can take encoded values A-D
#'
#' mycounts <- table(obsdata)
#' mycounts
#' calcEntropy(mycounts)
#'
#' mycounts <- table(factor(obsdata, levels = c('A', 'B', 'C', 'D')))
#' mycounts
#' calcEntropy(mycounts) # note that entropy estimate does not change, but max entropy and subsequently redundancy estimates will, with inclusion of empty bins
#'
#' # Example 2: Bivariate Data
#'
#' jointcount <- matrix(0, nrow = 4, ncol = 4)
#' jointcount[1,1] <- 1
#' jointcount[1,2] <- 1
#' jointcount[2,3] <- 1
#' jointcount[2,4] <- 1
#' jointcount[3,1] <- 1
#' jointcount[3,3] <- 1
#' jointcount[4,2] <- 1
#' jointcount[4,4] <- 1
#' jointcount
#'
#' calcEntropy(jointcount)
#'
#' # Example 3: Trivariate Data (from James & Crutchfield 2016)
#'
#' X <- c(0,1,0,1,2,3,2,3)
#' Y <- c(0,1,2,3,0,1,2,3)
#' Z <- c(0,1,2,3,2,3,0,1)
#'
#' counts <- table(X,Y,Z)
#' calcEntropy(counts)
#'
#'
#' @export


calcEntropy <- function(counts, logbase = 2, estimator = 'ML', isfreq = F, verbose = F, ...){

  if(estimator == 'CS'){
    if(logbase == 2){
      myunit = 'log2'
    }else if(logbase == 10){
      myunit = 'log10'
    }else if(logbase == exp(1)){
      myunit = 'log'
    }else{
      if(estimator != 'ML'){
        stop('Error: CS estimator only supports log bases 2, 10, or e (natural log)')
      }
    }
  }

  # format as matrix/3d array
  if(!is.matrix(counts) & !is.array(counts)){
    if(length(dim(counts)) < 3){
      dat <- as.matrix(counts)
    }else{
      dat <- array(as.matrix(counts),
                   dim = dim(counts),
                   dimnames = dimnames(counts))
    }
  }else{
    dat <- counts
  }

  # convert to frequencies

  if(isfreq & estimator == 'ML'){
    p = dat
  }else if(estimator == 'ML'){
    p <- dat/sum(dat)
  }else if(estimator == 'shrink'){

    if( sum(dim(dat)>1) == 1 ){
      p <- as.numeric(entropy::freqs.shrink(dat, verbose = F)) # get rid of lambda attrib attached to vector
    }else{
      p <- entropy::freqs.shrink(dat, verbose = F)
    }

  }else if(estimator %in% c("Jeffreys","Laplace", "SG", "minmax")){

    if(estimator == "Jeffreys"){
      a = 1/2
    }else if(estimator == 'Laplace'){
      a = 1
    }else if(estimator == 'SG'){
      a = 1/length(as.vector(dat))
    }else if(estimator == "minmax"){
      a = sum(dat)^0.5 / length(as.vector(dat))
    }else{
      stop('Error: Estimator method not supported')
    }

    p <- entropy::freqs.Dirichlet(dat, a = a)

  }else{
    stop('Error: Estimator method not supported')
  }




  if( sum(dim(dat)>1) == 1 ){ # 1D entropy

    if(sum(dat) == 0){ # if count vector is empty, set H to 0
      H <- 0
    }else{

      if(estimator == 'CS'){
        H <- as.numeric(entropy::entropy(dat, unit = myunit, method = 'CS', verbose = F))
      }else{
        H <- sum(p[p!=0] * log(1/p[p!=0], base = logbase))
      }

    }

    maxH <- log(length(dat), base = logbase)

    if(verbose){
      pout = p
      mylogbase = logbase
      myestimator = estimator
    }else{
      pout = NULL
      mylogbase = NULL
      myestimator = NULL
    }

    return(list(Entropy = H,
                MaxEntropy = maxH,
                Redundancy = 1 - H/maxH,
                estimator = myestimator,
                logbase = mylogbase,
                p = pout
                ))

  }else if( sum(dim(dat)>1) == 2 ){

    # marginal entropy
    margC <- calcEntropy( apply(p, 2, sum), logbase = logbase, isfreq = T)
    margR <- calcEntropy( apply(p, 1, sum), logbase = logbase, isfreq = T)

    # joint entropy
    jointH <- calcEntropy(as.vector(p), logbase = logbase, isfreq = T)$Entropy

    # conditional entropy
    pcol <- apply(p, 2, sum)
    chrow = sum(pcol * apply(p, 2, function(x) calcEntropy(x/ifelse(sum(x)==0,1,sum(x)), logbase = logbase, isfreq = T)$Entropy) )

    prow <- apply(p, 1, sum)
    chcol = sum(prow * apply(p, 1, function(x) calcEntropy(x/ifelse(sum(x)==0,1,sum(x)), logbase = logbase, isfreq = T)$Entropy) )

    if(verbose){
      pout = p
      mylogbase = logbase
      myestimator = estimator
    }else{
      pout = NULL
      mylogbase = NULL
      myestimator = NULL
    }

    return(list(MarginalEntropyRow = margR,
                MarginalEntropyCol = margC,
                JointEntropy = jointH,
                ConditionEntropyRow = chrow,
                ConditionEntropyCol = chcol,
                MutualInformation = margR$Entropy - chrow,
                RelMI = (margR$Entropy - chrow)/jointH,
                estimator = myestimator,
                logbase = mylogbase,
                p = pout
                ))


  }else if( sum(dim(dat) > 1) == 3 ){

    ogdim <- dim(dat)

    MI_XY <- calcEntropy(apply(p,
                               c(1,2),
                               function(x) sum(x, na.rm = T)
                               ),
                         logbase = logbase,
                         isfreq = T)
    MI_XZ <- calcEntropy(apply(p,
                               c(1,3),
                               function(x) sum(x, na.rm = T)
                               ),
                         logbase = logbase,
                         isfreq = T
                         )
    MI_YZ <- calcEntropy(apply(p,
                               c(2,3),
                               function(x) sum(x, na.rm = T)
                               ),
                         logbase = logbase,
                         isfreq = T
                         )

    # Calculate H(X;Y:Z)

    H_XYZ <- calcEntropy(as.vector(p),
                         logbase = logbase,
                         isfreq = T)$Entropy


    # Calculate I(X;Y | Z)

    H_X_Z <- MI_XZ$ConditionEntropyRow #H(X|Z)

    datx_yz <- p
    dim(datx_yz) <- c(ogdim[1], ogdim[2] * ogdim[3])
    H_X_YZ <- calcEntropy(datx_yz,
                          logbase = logbase,
                          isfreq = T)$ConditionEntropyRow

    MCE_XY <- H_X_Z - H_X_YZ

    # Calculate I(X;Z | Y)

    H_X_Y <- MI_XY$ConditionEntropyRow

    MCE_XZ <- H_X_Y - H_X_YZ

    # Calculate I(Y;Z | X)

    datz_xy <- p
    dim(datz_xy) <- c(ogdim[1] * ogdim[2] , ogdim[3])
    H_Z_X <- MI_XZ$ConditionEntropyCol
    H_Z_XY <- calcEntropy(datz_xy,
                          logbase = logbase,
                          isfreq = T)$ConditionEntropyCol

    MCE_YZ <- H_Z_X - H_Z_XY

    # Mutual Information

    IXYZ <- MI_XY$MutualInformation - MCE_XY
    totcor <- MCE_XY + MCE_XZ + MCE_YZ + IXYZ


    if(verbose){
      pout = p
    }else{
      pout = NULL
    }

    return(list(JointEntropy = H_XYZ,
                MutualInformation = IXYZ,
                MCE_XY = MCE_XY,
                MCE_XZ = MCE_XZ,
                MCE_YZ = MCE_YZ,
                MarginalMI_XY = MI_XY,
                MarginalMI_XZ = MI_XZ,
                MarginalMI_YZ = MI_YZ,
                TotalCorrelation = totcor,
                MI_TotCor = ifelse(totcor == 0, 0, IXYZ/totcor),
                estimator = estimator,
                logbase = logbase,
                p = pout
                ))

  }else{
    print('Error: Sorry, more than 3 dimensions is above my pay grade')
  }

}
