#' Generate Visual Comparisons of Encoded Data
#'
#' \code{compareEncodings} generates visualizations that facilitate direct comparisons between two encodings. A count matrix is created to assess the distribution of the joint encoding. Visualizations are then extracted from clustering output and added to margins of the count matrix to allow for direct comparison of patterns underlying either encodings.
#'
#' @param encodeout1 User may either pass 1) list output from LIT clustering utilities (dmplot or encodePlot) or 2) an encoded variable formatted as a one-column data formated with rownames
#' @param encodeout2 User may either pass 1) list output from LIT clustering utilities (dmplot or encodePlot) or 2) an encoded variable formatted as a one-column data formated with rownames
#' @param colorby Optional argument. Value used to color the cells in count matrix of joint encoding. Default option is cell 'counts'. Alternatively cells may be colored by Pointwise Mutual Information values using the 'PMI' option.
#' @param simPMI Optional argument. When colorby 'PMI', simulation stratedgy used to evaluate significance of PMI values. By default color values are not thresholded. Simulation options include: a permutation-based null model 'NullPerm', a parametric multinomial null model 'NullParam', and a parametric multinomial confidence interval 'CI Param'
#' @param nsim Optional argument. When colorby 'PMI', number of simulations to perform to evaluate significance of PMI values. Defulat is 1000 simulations.
#' @param alpha Optional argument. When colorby 'PMI', significance threshold used to evaluate significance of PMI values. Default is alpha = 0.05.
#' @param logbase Optional argument. When colorby 'PMI', base of log used to calculate PMI values. Default is 2 (shannon's information content)
#' @param gradient_color_low Optional argument. Color for lower bound of scale used to color cell values. Default is 'white'
#' @param gradient_color_high Optional argument. Color for upper bound of scale used to color cell values. Default is 'steelblue'
#' @param gridcolor Optional argument. Color used for lines to outline count matrix. Default is 'black'
#' @param nacolor Optional argument. Default is to drop NA values
#' @param confmat_xlab Optional argument. Label of row margin of count matrix. Default is no label.
#' @param confmat_ylab Optional argument. Label of column margin of count matrix. Default is no label.
#' @param fontsize Optional argument. Font size of count values in count matrix cells. Default is 25.
#' @param imheight Optional argument. Height of exported image. Default is 30 in.
#' @param imwidth Optional argument. Width of exported image. Default is 30.
#' @param imres Optional argument. Resolution of exported image. Default is 300,
#' @param filename Optional argument. Name of file visualization is exported to. Default filename is 'ClusterComparison'. By default image will be created in working directory. File path may be added to name to modify directory image is exported to.
#' @param extraplot Optional argument. User may provide an additional ggplot object to display in upper-left grid of visualization.
#' @param returnplots Optional argument. Default is FALSE. If TRUE all component plots are returned in list as ggplot objects.
#' @param verbose Optional argument. Default FALSE. If TRUE list is returned with summary stats for the joint encoding.
#'
#' @details compareEncodings generates a count matrix of the jointly encoded encoding. If the user provides output from other LIT clustering utilities, then visualizations of the raw data are extracted added to the appropriate table margin, wherein the order of the heatmap dendrogram will match the index of the table margin. An encoding may also be passed as a data frame, in which case a ggplot histogram will be generated and added the the appropriate margin. The resulting data visualization will allow users to directly compare encoded patterns in the raw observations across datasets.
#'
#' By default the count matrix cells are colored by the cell count to further facilitate visual evaluation of the joint distribution. If there is not association between the two data streams, then it is expected that the frequency of each cell in the joint probability distribution will mirrior the product of the two marginal distribution for either variable. For empirically encoded data, however, cluster sizes can vary substancially, which can make if difficult to judge by eye were cells are under- or over- populated relative the marginal probabilities. The 'PMI' colorby option allows the user to color cells by pointwise mutual information values to directly compare observed cell frequencies against the frequency under the null to facilitate such visual comparisons.
#'
#' Where data may be under-sampled, empirical estimates of PMI can have poor precision, particularly for cells where the population frequency is quite low, which can confound direct visual comparison. The compareEncodings utility thus also allso users to threshold PMI values, only coloring a cell if simulations confirm that a PMI value is significantly higher or lower than 0 using. With the NullPerm simulation option, observed PMI value are compared against values simulated under the null that no significant association exists between the two variables, which is simulated by randomly permuting indexes of the encoded values before recomputing the count matrix (perserving the marginal distribution). With the NullParam simulation option, observed PMI values are compared against values simulated under the null that no significant association exists between the two variables, which is simulated redrawing from multinomial using the products of the observed marginal freuqncies after Dirchilets correction of the joint joints. Finally, with the CIParam option, a confidence interval is approximated about each observed PMI, wherein the multinomal with expected probabilities equal to the Dirichlet's correct joint empirical frequncies is used to simulate the observed count matrix.
#'
#' @return If verbose is FALSE, visualizations will be exported to jpeg files. If verbose is TRUE, a list object will also be returned containing
#'
#' \item{PlotList}{If returnplots is TRUE, a list of ggplot objects used to construct the construct the composite visualization. If returnplot is FASE will return NULL}
#' \item{counts}{Observed count matrix of the joint encoding}
#' \item{obsfreq}{Observed empirical frequencies of the joint encoding}
#' \item{dirchletmeans}{Dirichlet's corrected frequencies of the joint encoding}
#' \item{nullfreq}{Expected joint frequency under the null assumption that no association exists between the two axes, calculated as the outer product of the marginal frequencies}
#' \item{PMI}{Pointwise mutual information of the joint encodings}
#' \item{PMIThresholded}{Pointwise mutual information values thresholded such that statistically insignificant estimates are set to zero}
#' \item{simquantile_lowerbound}{lower quantile bound of the simulated PMI values}
#' \item{simquantile_upperbound}{upper quantile bound of the simulated PMI values}
#'
#' @examples
#'
#' library(labstats)
#' data(locomotor)
#'
#' locowide <- unstack(locomotor, dist~time)
#' rownames(locowide) <- paste('Rat', unique(locomotor$animal), sep = '')
#' names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
#'
#' auxrow <- unique(locomotor[, c('animal', 'drug')])
#' rownames(auxrow) <- paste('Rat', auxrow$animal, sep = '')
#' auxrow <- auxrow[,'drug', drop = F]
#' names(auxrow) <- 'Treatment'
#'
#' encodeout_ward <- encodePlot(locowide, dist(locowide), dist(t(locowide)),
#'                     hclust_method_r = "ward.D2",
#'                     n_rowclusters = 5, n_colclusters = 0,
#'                     plot_title = 'Rat Locomotion Pattern: Wards D2 Encoding',
#'                     annotation_row = auxrow)
#'
#' encodeout_single <- encodePlot(locowide, dist(locowide), dist(t(locowide)),
#'                     hclust_method_r = "single",
#'                     n_rowclusters = 5, n_colclusters = 0,
#'                     plot_title = 'Rat Locomotion Pattern: Single Encoding',
#'                     annotation_row = auxrow)
#'
#' compareEncodings(encodeout_ward$nrow_5_ncol_0,
#'                  encodeout_single$nrow_5_ncol_0)
#'
#' @export
compareEncodings <- function(encodeout1, encodeout2,
                             #varnames = c('X','Y'),
                             colorby = 'count', #PMI
                             simPMI = NULL,
                             #NullPerm, NullParam, CIParam
                             nsim = 1000,
                             alpha = 0.05,
                             logbase = 2,
                             gradient_color_low = 'white',
                             gradient_color_high = 'steelblue',
                             gridcolor = 'black',
                             nacolor = NA,
                             confmat_xlab = '',
                             confmat_ylab = '',
                             fontsize = 25,
                             imheight = 30,
                             imwidth = 30,
                             imres = 300,
                             filename = 'ClusterComparison',
                             extraplot = NULL,
                             returnplots = FALSE,
                             verbose = FALSE
){


  ### preprocess data frames ###

  if(is.data.frame(encodeout1)){

    temp <- list()
    temp$rowclust <- as.data.frame(encodeout1)
    names(temp$rowclust) <- 'X'
    temp$plot <- ggplot(na.omit(temp$rowclust), aes(X)) +
      geom_bar() +
      xlab('') + ylab('Count')
    encodeout1 <- temp

  }

  if(is.data.frame(encodeout2)){

    temp <- list()
    temp$rowclust <- as.data.frame(encodeout2)
    names(temp$rowclust) <- 'Y'
    temp$plot <- ggplot(na.omit(temp$rowclust), aes(Y)) +
      geom_bar() +
      xlab('') + ylab('Count')
    encodeout2 <- temp

  }

  ### calc count matrix ###

  rowclust1 <- as.data.frame(encodeout1$rowclust)
  names(rowclust1) <- 'X'
  rowclust2 <- as.data.frame(encodeout2$rowclust)
  names(rowclust2) <- 'Y'

  mergedat <- transform(merge(rowclust1, rowclust2, by = 0),
                        row.names=Row.names, Row.names=NULL)
  mergedat <- mergedat[complete.cases(mergedat), ]
  confmat <- table(mergedat[,1], mergedat[,2])

  pmat <- (confmat)/sum(confmat) # empirical ML frequencies
  dmmat <- (confmat + 1)/sum(confmat + 1) # convert to dierchlets mean to account for counts of zero


  ### Calculate PMI ###

  px <- apply(dmmat, 1, sum)
  py <- apply(dmmat, 2, sum)

  exptmat <- outer(px, py, FUN = '*') # null expected joint frequency

  PMI <- log(dmmat/exptmat, base = logbase)

  # PMI[is.infinite(PMI)] <- 0
  # PMI <- (confmat/sum(confmat)) - exptmat

  #  # Calculate penalized PMI values
  #
  #   penPMI <- matrix(NA, nrow = nrow(pmat), ncol = ncol(pmat))
  #   #ibeta <- function(x,a,b){ pbeta(x,a,b)*beta(a,b) }
  #   N <- sum(confmat)
  #   n_row <- apply(confmat,1,sum)
  #   n_col <- apply(confmat,2,sum)
  #   s <- 1 - alpha
  #   y <- (1-s)/2
  #
  #   for(i in 1:nrow(pmat)){
  #     for(j in 1:ncol(pmat)){
  #
  #       mynumerator <- thetaCI(y,
  #                              confmat[i,j],
  #                              n_row[i])$LowerBound
  #       mydenom <- thetaCI(y,
  #                              n_col[j],
  #                              N)$UpperBound
  #
  #       penPMI[i,j] <- log(mynumerator/mydenom, base = logbase)
  #       penPMI[is.infinite(penPMI)] <- 0
  #
  #     }
  #   }
  #
  #   penPMI



  ### Simulations  ###

  # obsexptmat <- outer(apply(confmat/sum(confmat), 1, sum),
  #                     apply(confmat/sum(confmat), 2, sum), FUN = '*')


  if(is.null(simPMI)){
    PMI_thresh <- PMI
    qlower = NULL
    qupper = NULL
  }else{

    simout <- array(NA, dim= c(length(px),length(py),nsim))

    if(simPMI == 'NullParam'){

      for(i in 1:nsim){

        confmat_temp <- matrix(rmultinom(n = 1,
                                         size = sum(confmat),
                                         prob = exptmat),
                               #prob = obsexptmat),
                               nrow = length(px),
                               ncol = length(py))

        dmmat_temp <- (confmat_temp + 1)/sum(confmat_temp + 1)
        px_temp <- apply(dmmat_temp, 1, sum)
        py_temp <- apply(dmmat_temp, 2, sum)
        exptmat_temp <- outer(px_temp, py_temp, FUN = '*')
        simout[ , ,i] <- log(dmmat_temp/exptmat_temp, base = logbase)

      }

      qlower <- apply(simout, c(1,2),
                      function(x) quantile(x,alpha/2))
      qupper <- apply(simout, c(1,2),
                      function(x) quantile(x,1-alpha/2))

      PMI_thresh <- PMI *  !(PMI <= qupper & PMI >= qlower)


    }else if(simPMI == 'NullPerm'){

      for(i in 1:nsim){

        tempdat <- mergedat
        tempdat[,1] <- sample(tempdat[,1]) # permute over row index to break any bivariate associations

        confmat_temp <- table(tempdat[,1], tempdat[,2])
        dmmat_temp <- (confmat_temp + 1)/sum(confmat_temp + 1)
        px_temp <- apply(dmmat_temp, 1, sum)
        py_temp <- apply(dmmat_temp, 2, sum)
        exptmat_temp <- outer(px_temp, py_temp, FUN = '*')
        simout[ , ,i] <- log(dmmat_temp/exptmat_temp, base = logbase)

      }

      qlower <- apply(simout, c(1,2),
                      function(x) quantile(x,alpha/2))
      qupper <- apply(simout, c(1,2),
                      function(x) quantile(x,1-alpha/2))

      PMI_thresh <- PMI *  !(PMI <= qupper & PMI >= qlower)


    }else if(simPMI == 'CIParam'){

      for(i in 1:nsim){
        confmat_temp <- matrix(rmultinom(n = 1,
                                         size = sum(confmat),
                                         prob = dmmat),
                               #prob = obsexptmat),
                               nrow = length(px),
                               ncol = length(py))

        dmmat_temp <- (confmat_temp + 1)/sum(confmat_temp + 1)
        px_temp <- apply(dmmat_temp, 1, sum)
        py_temp <- apply(dmmat_temp, 2, sum)
        exptmat_temp <- outer(px_temp, py_temp, FUN = '*')
        simout[ , ,i] <- log(dmmat_temp/exptmat_temp, base = logbase)

      }

      qlower <- apply(simout, c(1,2),
                      function(x) quantile(x,alpha/2))
      qupper <- apply(simout, c(1,2),
                      function(x) quantile(x,1-alpha/2))

      PMI_thresh <- PMI *  !(qlower <= 0  & qupper >= 0)
      PMI_thresh[PMI_thresh != 0 & qlower > 0] <- qlower[PMI_thresh != 0 & qlower > 0]
      PMI_thresh[PMI_thresh != 0 & qupper < 0] <- qupper[PMI_thresh != 0 & qupper < 0]



    }else{
      stop('Error: PMI simulation method not recognized')
    }

  }




  # Setup Matrix Viz

  temp1 <- reshape2::melt(confmat)
  rownames(temp1) <- paste(temp1$Var1, temp1$Var2, sep = '_')
  names(temp1)[which(names(temp1) == 'value')] <- 'Count'

  temp2 <- reshape2::melt(PMI_thresh)
  rownames(temp2) <- paste(temp2$Var1, temp2$Var2, sep = '_')
  names(temp2)[which(names(temp2) == 'value')] <- 'PMI'

  temp <- merge(temp1, temp2, by = 0)
  names(temp)[which(names(temp) == 'Var1.x')] <- 'Var1'
  names(temp)[which(names(temp) == 'Var2.x')] <- 'Var2'

  if(is.integer(temp$Var1)){
    temp$Var1 <- factor(temp$Var1,
                        levels = sort(unique(temp$Var1)),
                        ordered = T)
  }

  if(is.integer(temp$Var2)){
    temp$Var2 <- factor(temp$Var2,
                        levels = sort(unique(temp$Var2)),
                        ordered = T)
  }




  if(colorby == 'count'){

    matplot <- ggplot(temp, aes(x = Var1, y = Var2, fill = Count)) +
      geom_tile(colour = gridcolor) +
      #geom_text(aes(label = sprintf("%1.2f", Count)), vjust = 1) +
      geom_text(aes(label = Count), size = fontsize) +
      scale_fill_gradient(low = gradient_color_low,
                          high = gradient_color_high,
                          na.value = nacolor) +
      #xlim(mylevelsx) +
      ylim(rev(levels(temp$Var2))) +
      #scale_y_reverse() +
      xlab(confmat_xlab) + ylab(confmat_ylab) + theme_bw() +
      theme(panel.border = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            axis.ticks.x = element_blank(),
            #axis.text.x = element_blank(),
            axis.ticks.y = element_blank(),
            #axis.text.y = element_blank(),
            legend.position = "none",
            axis.text = element_text(size = fontsize))

  }else if(colorby == 'PMI'){

    matplot <- ggplot(temp, aes(x = Var1, y = Var2, fill = PMI)) +
      geom_tile(colour = gridcolor) +
      #geom_text(aes(label = sprintf("%1.2f", Count)), vjust = 1) +
      geom_text(aes(label = Count), size = fontsize) +
      scale_fill_gradient2(low = gradient_color_low,
                           mid = 'white',
                           high = gradient_color_high,
                           na.value = nacolor,
                           midpoint = 0) +
      ylim(rev(levels(temp$Var2))) +
      #scale_y_reverse() +
      xlab(confmat_xlab) + ylab(confmat_ylab) +  theme_bw() +
      theme(panel.border = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            axis.ticks.x = element_blank(),
            #axis.text.x = element_blank(),
            axis.ticks.y = element_blank(),
            #axis.text.y = element_blank(),
            #legend.position = "none",
            axis.text = element_text(size = fontsize))
  }else{
    stop('Error: Option for colorby not recognized')
  }


  # Create joint viz


  if(sum(class(encodeout1$plot) == 'list') == 1){
    grid1 = 1:length(encodeout1$plot)
  }else{
    grid1 = 1
  }

  if(sum(class(encodeout2$plot) == 'list') == 1){
    grid2 = 1:length(encodeout2$plot)
  }else{
    grid2 = 1
  }

  for(i in grid1){
    for(j in grid2){

      if(sum(class(encodeout1$plot) == 'list') == 1){
        xheat <- ggplotify::as.ggplot(encodeout1$plot[[i]]) +
          theme(legend.position = "none")
        xname <- names(encodeout1$plot)[i]
      }else{
        xheat <- ggplotify::as.ggplot(encodeout1$plot) +
          theme(legend.position = "none")
        xname <-''
      }

      if(sum(class(encodeout2$plot) == 'list') == 1){
        yheat <- ggplotify::as.ggplot(encodeout2$plot[[j]]) +
          theme(legend.position = "none")
        yname <- names(encodeout2$plot)[j]
      }else{
        yheat <- ggplotify::as.ggplot(encodeout2$plot) +
          theme(legend.position = "none")
        yname <- ''
      }

      if(is.null(extraplot)){
        plotout <- ggpubr::ggarrange(NULL,
                                     ggplotify::as.ggplot(xheat),
                                     ggplotify::as.ggplot(yheat),
                                     matplot,
                                     ncol = 2, nrow = 2)
      }else{
        plotout <- ggpubr::ggarrange(
          ggplotify::as.ggplot(extraplot),
          ggplotify::as.ggplot(xheat),
          ggplotify::as.ggplot(yheat),
          matplot,
          ncol = 2, nrow = 2)
      }

      # don't export pdf if filename is set to NA or NULl
      if( !(is.na(filename) | is.null(filename)) ){
        ggpubr::ggexport(plotout,
                         width = imwidth,
                         height = imheight,
                         res = imres,
                         filename = paste(filename,
                                          xname,
                                          yname,
                                          '.pdf', sep = ''))
      }

    }
  }

  if(returnplots){
    rplotlist <- list(
      fullplot = plotout,
      encoding1plot = ggplotify::as.ggplot(xheat),
      encoding2plot = ggplotify::as.ggplot(yheat),
      countnmatrix = matplot
    )
  }else{
    rplotlist <- NULL
  }

  if(verbose | returnplots){
    return(list(PlotList = rplotlis,
                counts = confmat,
                obsfreq = pmat,
                dirchletmeans = dmmat,
                nullfreq = exptmat,
                PMI = PMI,
                PMIThresholded = PMI_thresh,
                simquantile_lowerbound = qlower,
                simquantile_upperbound = qupper
    ))
  }

  # xheat <- ggplotify::as.ggplot(encodeout1$plot) +
  #   theme(legend.position = "none")
  # yheat <- ggplotify::as.ggplot(encodeout2$plot) +
  #   theme(legend.position = "none")
  #

  # plotout <- ggpubr::ggarrange(NULL,
  #                              ggplotify::as.ggplot(xheat),
  #                              ggplotify::as.ggplot(yheat), matplot,
  #                              ncol = 2, nrow = 2) #,  align = "hv", widths = c(2, 1), heights = c(1, 2))
  #
  # ggpubr::ggexport(plotout,
  #                  width = imwidth, height = imheight, res = imres,
  #                  filename = paste(filename, '.pdf', sep = ''))

}
