#' Recursively Test for Branch Significance

#' \code{cutreeEnsemble} applies the testBranch utility recusively through a dendrogram, working down each branch and pruning the tree where it detects the signal has fallen below the resolution of the data set based simulated error model in the ensemble of mimicked data sets.
#'
#' @param obsdat Data matrix or array with observational units on the row axis. If data is generated through repeated sampling, time will be indexed on the second index, and features on the third and final axis. If data is not time indexed, features should be on the column axis
#' @param simdata Ensemble of data matrices simulated under the alternative hypothesis. Axes indices should match that observed data, with simulation index on the final axis.
#' @param rawdata Data array containing raw observations prior to any temporal aggregation.
#' @param feature_subset Optional argument. Default is to use all features in calculating dissimilarity values. User may pass a vector of feature names to calculate dissimilarity values for only a subset of features.
#' @param dist_method Optional argument. Dissimilarity estimate used to encode data. Options are 'Euclidean', 'KLD, or 'Ensemble'. Default is Euclidean.
#' @param weightdist Optional argument. Precomputed weight values that may be used to calculate ensemble weighted distances. See ensembleDist for more details
#' @param hclust_method Optional argument. Agglomeration method employed to create dendrogram. Default is 'ward.D2'. See hclust for more options.
#' @param nclustalt Optional integer argument. By default, if branch test with binary encoding is insignificant, branch will be pruned. User may elect to re-test branch with an alternative encoding granularity >2.
#' @param logbase Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).
#' @param SimSpecs Optional argument. Structured list containing all specifications needed to parameterize a simulated time budget. See simTimeBudget for formatting details
#' @param maxiter Optional argument. Maximum size that simulation under the alternative may be extended. Default is 2000. May be set to size of presimulated data set to prevent additional simulations from being performed to resolve ambiguous branch tests
#' @param verbose Optional logical argument. Default is FALSE. Should recursion results return more comprehensive simulation reslts. See testBranch for details.
#' @param showmSimPlots Optional logical argument. Default is FALSE. Should visualizations of the testBranch results for each evaluated branch be returned to screen.
#' @param showDendrogram Optional logical argument. Default is TRUE. Should a dendrogram visualization with branch colors set to reflect final pruning results be returned to screen.
#' @param showHeatmap Optional logical argument. Default is TRUE. Should a heatmap visualization with gaps between rows to reflect pruned branches be returned to screen.
#' @param exportPlots Optional logical argument. Default is FALSE. Should dendrogram and heatmap visualizations be exported as image files.
#' @param filename Optional argument. If exportPlots is TRUE, file name and location for image export.
#' @param imSpecDend Optional argument. If exportPlots is TRUE, list with specification for imwidth, imheight, and imres of dendrogram image. Default is list(imwidth = 10, imheight = 20, imres = 300)
#' @param imSpecHeatmap Optional argument. If exportPlots is TRUE, list with specification for imwidth, imheight, and imres of heatmap image. Default is list(imwidth = 12, imheight = 18, imres = 300)
#' @param ... additional plotting options may be passed directly to pheatmap plot
#'
#' @details When cutting/prunning a tree to generate clusters (ie - an encoding), the ultimate goal is to pick the point where where signal ends and we get into the stochastic weeds below the intrinsic resoluation of the data. When we do this by visual inspection, we look for a sudden drop off in branch length, but such decisions can be subjective and poorly replicable. More often trees are pruned at a fixed branch height, but for data sets whose metrics are distributed with nonhomogenous variance, branch lengths cannot be directly compared.
#'
#' Ensemble cut recursively tests the statistical significance of each branch using the testBranch utility. This more localized approach allows for a more adaptive pruning that can accomodate more complex error structures without uncessesary loss of information in the final encoding.
#'
#' @return A structured list containing:
#'
#' \item{SimResults}{A list containing testBranch results for each branch tested}
#' \item{FinalEncoding}{Vector containing discretely encoded observations based on the pruned dendrogram}
#' \item{Dendrogram}{hclust object containing the complete tree for the observed data set}
#' \item{DendrogramPlot}{ggplot object containing visualization of the complete tree for the observed data set, wherein branchess are colored to reflect the final pruning}
#' \item{Heatmap}{pheatmap visualization of the observed data with gaps along the row axis to indicate the pruned branches}
#'
#'
#'
#' @export



cutreeEnsemble <- function(obsdat, simdata, rawdata, nclustalt = NULL, B = 2000, maxiter = 2000, feature_subset = NULL, hclust_method = 'ward.D2', dist_method = 'Euclidean', weightdist = NULL, SimSpecs = NULL, logbase = 2, showmSimPlots = FALSE, verbose = F, showDendrogram = TRUE, showHeatmap = T, exportPlots = F, filename = 'EnsembleCutPlot', imSpecDend = list(imwidth = 10, imheight = 20, imres = 300), imSpecHeatmap = list(imwidth = 12, imheight = 18), imres = 300, ...){ # dendColorPalette ='Paired'

  require(dendextend)
  require(RColorBrewer)
  #require(DirichletReg)

  # if(is.null(feature_subset)){
  #   feature_subset = names(obsdat)
  # }

  if(!is.null(feature_subset)){
    obsdat <- obsdat[ , feature_subset]
    simdata <- simdata[,feature_subset,]
    rawdata <- rawdata[ , , feature_subset]
    if(!is.null((weightdist))){
      weightdist <- weightdist[ , feature_subset]
    }
  }

  rout <- recursiveTestBranch(obsdat,  # obsdat[temp,],
                              simdata, # simdata[temp,,],
                              rawdata, # rawdata[temp,,],
                              nclustalt = nclustalt,
                              maxiter = maxiter,
                              B = B,
                              feature_subset = feature_subset,
                              hclust_method = hclust_method,
                              logbase = logbase,
                              showplot = showmSimPlots,
                              verbose = verbose,
                              dist_method = dist_method,
                              SimSpecs = SimSpecs,
                              weightdist = weightdist)

  # return vector of cluster ID's numbered in order returned by recursive

  cutout <- data.frame(ObsID = NA,
                       ClusterID = NA,
                       stringsAsFactors = F)
  mylevels <- 1
  if(length(rout) > 1){
    for(i in 1:length(rout)){

      if(rout[[i]]$Type == 'Cluster'){

        # if(sum(is.na(rout[[i]]$LeafID))){print(i)}

        cutout <- rbind(cutout,
                        data.frame(ObsID = rout[[i]]$LeafID,
                                   ClusterID = rep(mylevels,
                                                   length(rout[[i]]$LeafID)),
                                   stringsAsFactors = F)
        )
        mylevels <- mylevels + 1
      }

    }
  }else{
    cutout$ClusterID <- 1
  }



  cutout <- cutout[-1, ]
  rownames(cutout) <- cutout$ObsID
  #print(sum(is.na(rownames(cutout))))

  clusterout_data <- cutout[match(rownames(obsdat), rownames(cutout)), ]
  #print(sum(is.na(clusterout_data$ClusterID)))



  # Reassign cluster ID values to match order of dendrogram

  htreeout <- getBinaryCode(obsdat,
                            simdata = simdata,
                            dist_method = dist_method,
                            hclust_method = hclust_method,
                            returnTree = T
  )$hctree
  hout <- as.dendrogram(htreeout)
  #print(labels(hout))
  #print(sum(is.na(labels(hout))))


  #print(unique(clusterout_data$ClusterID))
  temp <- clusterout_data[order.dendrogram(hout),] # data ordered to mach dendrogram lables
  ctemp <- unique(temp$ClusterID)
  #print(ctemp)

  clusterout_data$DendID <- NA

  for (i in 1:length(ctemp)){
    clusterout_data$DendID[clusterout_data$ClusterID == ctemp[i]] <- i
  }
  #print(sum(is.na(clusterout_data$DendID)))


  # Dendrogram Viz

  temp <- clusterout_data[order.dendrogram(hout),] # data ordered to mach dendrogram lables
  col <- rainbow(length(ctemp))
  #col <- brewer.pal(length(ctemp), dendColorPalette)

  for(i in 1:length(ctemp)){
    hout <- set(hout, 'by_labels_branches_col',
                value = temp$ObsID[temp$DendID == i],
                TF_values = c(col[i],Inf))
  }

  dendout <- as.ggdend(hout)
  dendout <- ggplot(dendout, theme = theme_minimal(), horiz = T)
  if(showDendrogram){
    plot(hout)
  }


  # Heatmap Viz

  temp <- clusterout_data[order.dendrogram(hout),] # data ordered to mach dendrogram lable
  mygaps <- c()
  for(i in 1:(length(table(temp$DendID)) - 1) ){
    mygaps <- c(mygaps, max(which(temp$DendID == i)))
  }

  plotout <- pheatmap::pheatmap(
    mat = obsdat[order.dendrogram(hout),],
    cluster_rows = F,
    #clustering_method = "ward.D2",
    #annotation_row = auxmat_tbvar,
    #annotation_colors  = auxcol_var,
    gaps_row = mygaps,
    silent = T,
    ...)

  if(showHeatmap){
    plot.new()
    print(plotout)
  }


  # Export Plots

  if(exportPlots){
    ggpubr::ggexport(dendout ,
                     width = imSpecDend$imwidth,
                     height = imSpecDend$imheight,
                     res = imres,
                     filename = paste(filename,
                                      '_Dendrogram',
                                      '.pdf', sep = ''))
    ggpubr::ggexport(plotout,
                     width = imSpecHeatmap$imwidth,
                     height = imSpecHeatmap$imheight,
                     res = imres,
                     filename = paste(filename,
                                      '_Heatmap',
                                      '.pdf', sep = ''))
  }



  # Return Info

  outdat <- clusterout_data$DendID
  names(outdat) <- clusterout_data$ObsID

  return(list(SimResults = rout,
              FinalEncoding = outdat,
              Dendrogram = hout,
              DendrogramPlot = dendout,
              Heatmap = plotout))

}
