#' Generate a data mechanics plot.
#'
#' \code{dmplot} performs a data mechanics clustering on the row and column axes of a data frame and displays the results using the pheatmap packaging plotting options
#'
#' @param obsdata A data frame with observations on the row axis and features on the column axis. Where features differ in magnitude, users are advised to normalized by column.
#' @param n_rowclusters Scaler value indicating the number of clusters used to partition row axis
#' @param n_colclusters Scaler value indicating the number of clusters used to partition column axis
#' @param hclust_method Optional arguement. String indicating the clustering method used to creat hc tree. Default is Ward D2. See hclust for additional options.
#' @param maxreps Optional arguement. Maximum number of interations used to converge towards final clustering.
#' @param verbose Optional arguement. TRUE/FALSE. Should iteration results and column/row distance matrix be included in output list. Defualt is FALSE.
#' @param showplot_final Optional arguement. TRUE/FALSE. Should final plot be returned to screen. Default is TRUE
#' @param showplot_all Optional arguement. TRUE/FALSE. Should plots of intermediate clustering results be returned to screen. Default if FALSE
#' @param plot_title Optional arguement. Title of data mechanics plot.
#' @param export_plot_final Optional arguement. TRUE/FALSE. Should final plot be exported to jpg to current working directors. Default is FALSE
#' @param export_plot_all Optional arguement. TRUE/FALSE. Should intermediate plots before clustering converges be exported to jpg to current working directory. Default is FALSE
#' @param imwidth Optional arguement. Width of exported jpeg plot.
#' @param imheight Optional arguement. Height of exprted jpeg plot.
#' @param imres Optional arguement. Resolution of exported jpeg.
#' @param filename Optional arguement. File path and name of data mechanics plots exported to jpeg.
#' @param rorderdend Optional arguement. Should cluster numbers for final row clusters match the order of the dendrogram or order of appearance in data. Default is order of dendrogram. (Note - Iteration results will order cluster numbers by data as default stats::cutree)
#' @param corderdend Optional arguement. Should cluster numbers for final col clusters match the order of the dendrogram or order of appearance in data. Default is order of dendrogram. (Note - Iteration results will order cluster numbers by data as default stats::cutree)
#' @param ... Aditional plotting options passed directly to pheatmap.
#'
#' @return Returns a list containing
#' \item{plot}{Heatmap plot where row and column orderings and divisions are determined by ouput of data mechnics optimizations}
#' \item{coltree}{hclust trees corresponding to the column axis}
#' \item{rowtree}{hclust trees corresponding to the row axis}
#' \item{colclust}{vectors with clustering assigmnets to each feature column in the order provided in obsdata}
#' \item{rowclust}{vectors with clustering assigmnets to each observation row in the order provided in obsdata}
#' \item{coldist}{distance matrices generated by data mechanics optimization to create the row and column trees}
#' \item{coldist}{distance matrices generated by data mechanics optimization to create the column tree}
#' \item{rowdist}{distance matrices generated by data mechanics optimization to create the row tree}
#' \item{iteration_results}{list containing intermediate clustering results and plots prior convergence of data mechanics optimization}
#'
#' @examples
#' library(labstats)
#' data(locomotor)
#'
#' trtrats <- locomotor[locomotor$drug == 'Drug',]
#' locowide <- unstack(trtrats, dist~time)
#' rownames(locowide) <- paste('Rat', unique(trtrats$animal), sep = '')
#' names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
#'
#' dmout <- dmplot(locowide, n_rowclusters = 3, n_colclusters = 4,
#'                 showplot_all = F, showplot_final = T,
#'                 plot_title = 'Rat Data Mechanics Plot')
#'
#'
#' locowide <- unstack(locomotor, dist~time)
#' rownames(locowide) <- paste('Rat', unique(locomotor$animal), sep = '')
#' names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
#' auxrow <- unique(locomotor[, c('animal', 'drug')])
#' rownames(auxrow) <- paste('Rat', auxrow$animal, sep = '')
#' auxrow <- auxrow[,'drug', drop = F]
#' names(auxrow) <- 'Treatment'
#'
#' dmout <- dmplot(locowide, n_rowclusters = 3, n_colclusters = 4,
#'                 showplot_all = F, showplot_final = T,
#'                 plot_title = 'Rat Data Mechanics Plot',
#'                 annotation_row = auxrow)
#'
#' @export


dmplot <- function(obsdata, n_rowclusters = 1, n_colclusters = 1, hclust_method = "ward.D2", maxreps = 10, verbose = FALSE, showplot_final = TRUE, showplot_all = FALSE, plot_title = 'Data Mechanics Plot', export_plot_final = FALSE, export_plot_all = FALSE, imwidth = 10, imheight = 15, imres = 300,  filename = 'DatMechPlotOut', rorderdend = TRUE, corderdend = TRUE, ...){

  # require("pheatmap")
  # require("RColorBrewer")
  # require('dplyr')

  # prepare data

  clusters.col.out <- list()
  clusters.row.out <- list()

  hclust_row_out <- list()
  hclust_col_out <- list()

  plots_out <- list()

  # initial cluster

  hclustr <- stats::hclust(dist(obsdata), method = hclust_method)
  hclust_row_out[[1]] <- hclustr
  clusters.row.out[[1]] <- stats::cutree(hclustr, n_rowclusters)

  hclustc <- stats::hclust(dist(t(obsdata)), method = hclust_method)
  hclust_col_out[[1]] <- hclustc
  clusters.col.out[[1]] <- stats::cutree(hclustc, n_colclusters)

  plots_out[[1]] <- pheatmap::pheatmap(mat = obsdata,
                             cluster_cols = hclustc,
                             cluster_rows = hclustr,
                             #clustering_method = "ward.D2",
                             cutree_rows = n_rowclusters ,
                             cutree_cols = n_colclusters ,
                             #annotation_row = auxdata,
                             main = paste(plot_title, ': Iteration 0', sep = ''),
                             silent = ifelse(showplot_all, F, T), # if showplot_all = FALSE then don't print plot
                             ...)

  if(export_plot_all){
    jpeg(paste(filename,'_V0.jpeg'), width = imwidth, height = imheight, units = 'in', res = imres)
      print(plots_out[[1]])
    dev.off()
  }


  # recalculate row distances using DM

  rowdist.new = matrix(0, nrow(obsdata), nrow(obsdata))
  rowdist.list = list()
  rowdim.list = numeric(n_colclusters)
  for(i in 1:n_colclusters){
    #i = 1
    #print(i)
    inds = which(stats::cutree(hclustc, n_colclusters) == i)
    temp = obsdata[,inds]
    tempmat <- as.matrix(dist(temp))/sqrt(length(inds))
    tempmat[is.na(tempmat)] <- 0 # if any NA distances due to missing values, set distance to 0
    rowdist.new = rowdist.new + tempmat  #This is the calculation for updating row distances using the original column hierarchical clustering tree.  The denominator normalizes the weigths so large clusters don't have disproportionate influence on the new distance matrix.
    #print(sum(sum(is.na(rowdist.new))))
    rowdist.list[[i]] = as.matrix(dist(temp))
    rowdim.list[i] = length(inds)
  }

  hclustr <- stats::hclust(as.dist(rowdist.new), method = hclust_method)
  hclust_row_out[[length(hclust_row_out)+1]] <- hclustr
  clusters.row.out[[length(clusters.row.out) + 1]] <- stats::cutree(hclustr, n_rowclusters)


  # Recalculate column distances

  coldist.new = matrix(0, ncol(obsdata), ncol(obsdata))
  #coldist.new = as.matrix(dist(t(obsdata)))
  for(i in 1:n_rowclusters){
    # i = 1
    inds = which(stats::cutree(hclustr, n_rowclusters) == i)
    temp = obsdata[inds,]
    tempmat <- as.matrix(dist(t(temp)))/sqrt(length(inds))
    tempmat[is.na(tempmat)] <- 0
    coldist.new = coldist.new + tempmat
  }

  hclustc <- stats::hclust(as.dist(coldist.new), method = hclust_method)
  hclust_col_out[[length(hclust_col_out)+1]] <- hclustc
  clusters.col.out[[length(clusters.col.out)+1]] <- stats::cutree(hclustc, n_colclusters)

  plots_out[[length(plots_out)+1]] <- pheatmap::pheatmap(mat = obsdata,
                                              cluster_cols = hclustc,
                                              cluster_rows = hclustr,
                                              #clustering_method = "ward.D2",
                                              cutree_rows = n_rowclusters ,
                                              cutree_cols = n_colclusters ,
                                              #annotation_row = auxdata,
                                              main = paste(plot_title, ': Iteration 1', sep = ''),
                                              silent = ifelse(showplot_all, F, T),
                                              ...)

  if(export_plot_all){
    jpeg(paste(filename,'_V1.jpeg'), width = imwidth, height = imheight, units = 'in', res = imres)
      print(plots_out[[length(plots_out)]])
    dev.off()
  }

  # Loop through this process for a while until cluster membership doesn't change, or until counter reaches a prespecified number.

  ctr = 2
  while(ctr <= maxreps){

    #print(ctr)

    # recalculate row distances us DM

    rowdist.new = matrix(0, nrow(obsdata), nrow(obsdata))
    rowdist.list = list()
    rowdim.list = numeric(n_colclusters)
    for(i in 1:n_colclusters){
      #i = 1
      inds = which(stats::cutree(hclustc, n_colclusters) == i)
      temp = obsdata[,inds]
      tempmat <- as.matrix(dist(temp))/sqrt(length(inds))
      tempmat[is.na(tempmat)] <- 0
      rowdist.new = rowdist.new + tempmat  #This is the calculation for updating row distances using the original column hierarchical clustering tree.  The denominator normalizes the weigths so large clusters don't have disproportionate influence on the new distance matrix.
      rowdist.list[[i]] = as.matrix(dist(temp))
      rowdim.list[i] = length(inds)
    }

    hclustr <- stats::hclust(as.dist(rowdist.new), method = hclust_method)
    hclust_row_out[[length(hclust_row_out)+1]] <- hclustr
    clusters.row.out[[length(clusters.row.out) + 1]] <- stats::cutree(hclustr, n_rowclusters)


    # Recalculate column distances

    coldist.new = matrix(0, ncol(obsdata), ncol(obsdata))
    #coldist.new = as.matrix(dist(t(obsdata)))
    for(i in 1:n_rowclusters){
      # i = 1
      inds = which(stats::cutree(hclustr, n_rowclusters) == i)
      temp = obsdata[inds,]
      tempmat <- as.matrix(dist(t(temp)))/sqrt(length(inds))
      tempmat[is.na(tempmat)] <- 0
      coldist.new = coldist.new + tempmat
    }

    hclustc <- stats::hclust(as.dist(coldist.new), method = hclust_method)
    hclust_col_out[[length(hclust_col_out)+1]] <- hclustc
    clusters.col.out[[length(clusters.col.out) + 1]] <- stats::cutree(hclustc, n_colclusters)

    plots_out[[length(plots_out)+1]] <- pheatmap::pheatmap(mat = obsdata,
                                                 cluster_cols = hclustc,
                                                 cluster_rows = hclustr,
                                                 #clustering_method = "ward.D2",
                                                 cutree_rows = n_rowclusters ,
                                                 cutree_cols = n_colclusters ,
                                                 #annotation_row = auxdata,
                                                 main = paste(plot_title, ': Iteration ', ctr, sep = ''),
                                                 silent = ifelse(showplot_all, F, T),
                                                 ...)

    if(export_plot_all){
      jpeg(paste(filename, '_V',ctr,'.jpeg', sep = '') , width = imwidth, height = imheight, units = 'in', res = imres)
        print(plots_out[[length(plots_out)]])
      dev.off()
    }

    if(
      sum((clusters.col.out[[length(clusters.col.out)]] - clusters.col.out[[length(clusters.col.out) - 1]])^2) == 0 &
      sum((clusters.row.out[[length(clusters.row.out)]] - clusters.row.out[[length(clusters.row.out) - 1]])^2) == 0
    ){ ctr = 1000000 } # bottom out if the row and column clusters don't change

    ctr = ctr + 1


  }

  if (rorderdend) {
    finaltree <- hclust_row_out[[length(hclust_row_out)]]
    cutout <- dendextend::cutree(as.dendrogram(finaltree),
                                 n_rowclusters, order_clusters_as_data = F)
    names(cutout) <- rownames(obsdata)[dendextend::order.hclust(finaltree)]
    finalrowcluster <- cutout[match(rownames(obsdata), names(cutout))]
  }
  else {
    finalrowcluster <- clusters.row.out[[length(clusters.row.out)]]
  }
  if (corderdend) {
    finaltree <- hclust_col_out[[length(hclust_col_out)]]
    cutout <- dendextend::cutree(as.dendrogram(finaltree),
                                 n_colclusters, order_clusters_as_data = F)
    names(cutout) <- colnames(obsdata)[dendextend::order.hclust(finaltree)]
    finalcolcluster <- cutout[match(colnames(obsdata), names(cutout))]
  }
  else {
    finalcolcluster <- clusters.col.out[[length(clusters.col.out)]]
  }

  plot_final <- pheatmap::pheatmap(mat = obsdata,
                         cluster_cols = hclustc,
                         cluster_rows = hclustr,
                         #clustering_method = "ward.D2",
                         cutree_rows = n_rowclusters ,
                         cutree_cols = n_colclusters ,
                         #annotation_row = auxdata,
                         main = plot_title,
                         silent = ifelse(showplot_final, F, T),
                         ...)

  if(export_plot_final){
    jpeg(paste(filename,'.jpeg'), width = imwidth, height = imheight, units = 'in', res = imres)
      print(plot_final)
    dev.off()
  }

  # if(showplot_all){
  #   for(p in 1:length(plots_out)){
  #     print(plots_out[[p]])
  #   }
  # }else{
  #   if(showplot_final){
  #     print(plot_final)
  #   }
  # }

  if(verbose){

    iteration_results <- list(col_clusters = clusters.col.out,
                              row_clusters = clusters.row.out,
                              col_trees = hclust_col_out,
                              row_trees = hclust_row_out,
                              plots = plots_out)
    return(list(plot = plot_final,
                colclust = finalcolcluster,
                rowclust = finalrowcluster,
                coltree = hclust_col_out[[length(hclust_col_out)]],
                rowtree = hclust_row_out[[length(hclust_row_out)]],
                coldist = as.dist(coldist.new),
                rowdist = as.dist(rowdist.new),
                iterationresults = iteration_results
    ))

  }else{

    return(list(plot = plot_final,
                colclust = finalcolcluster,
                rowclust = finalrowcluster,
                coltree = hclust_col_out[[length(hclust_col_out)]],
                rowtree = hclust_row_out[[length(hclust_row_out)]],
                coldist = NULL,
                rowdist = NULL,
                iterationresults = list()
    ))

  }








}
