#' Generate a data mechanics plot on a grid of row and column cluster numbers
#'
#' \code{dmplotGrid} generates data mechanics plots for each combination of row and column cluster numbers provided in ranges by the user.
#'
#' @param obsdata A data frame with observations on the row axis and features on the column axis. Where features differ in magnitude, users are advised to normalized by column.
#' @param n_rowclusters Vector containing a range or set of scalar values for the number of clusters used to partition row axis
#' @param n_colclusters Vector containing a range or set of scalar values for the number of clusters used to partition column axis
#' @param hclust_method Optional arguement. String indicating the clustering method used to creat hc tree. Default is Ward D2. See hclust for additional options.
#' @param maxreps Optional arguement. Maximum number of interations used to converge towards final clustering.
#' @param verbose Optional arguement. TRUE/FALSE. Should iteration results and column/row distance matrix be included in output list. Defualt is FALSE.
#' @param showplots Optional arguement. TRUE/FALSE. Should final plot for each combination of column and row cluster values be returned to screen. Default is FALSE.
#' @param plot_title Optional arguement. Title of data mechanics plot. Should final plot for each combination of column and row cluster values be
#' @param export_plots Optional arguement. TRUE/FALSE. Should final plot for each combination of column and row cluster values be  exported to jpg to current working directors. Default is TRUE
#' @param imwidth Optional arguement. Width of exported jpeg plot.
#' @param imheight Optional arguement. Height of exprted jpeg plot.
#' @param imres Optional arguement. Resolution of exported jpeg.
#' @param filename Optional arguement. File path and name of data mechanics plots exported to jpeg.
#' @param ... Aditional plotting options passed directly to pheatmap.
#'
#' @return Returns a named list containing output for each data mechanic optimization performed on the grid of metaparameters, indexed by row and column numbers. See \code{dmplot} for details of elements in list of output provided by each data mechanics optimization.
#'
#' @export



dmplotGrid <- function(obsdata, n_rowclusters = 1, n_colclusters = 1, hclust_method = "ward.D2", maxreps = 10, verbose = FALSE, showplots = FALSE, plot_title = 'Data Mechanics Plot', export_plots = TRUE, imwidth = 10, imheight = 15, imres = 300, filename = 'DatMechPlotOut', ...){


  cluster.results <- list()

  for(r in n_rowclusters){
    for(c in n_colclusters){

      cout <- dmplot(obsdata, n_rowclusters = r, n_colclusters = c, hclust_method = hclust_method, maxreps = maxreps, verbose = verbose, showplot_final = showplots, export_plot_final = export_plots, imwidth = imwidth , imheight = imheight, imres = imres , plot_title = paste(plot_title, r, 'Rows', c ,'Cols', sep = ' '), filename = paste(filename,'_R', r, 'C', c, sep = ''), ...)

      if(showplots){
        print(cout$plot)
      }

      cluster.results[[paste('nrow_',r,'_ncol_',c, sep = '')]] <- cout

    }
  }

  return(cluster.results)

}
