#' Encode Data & Visualize Heatmap
#'
#' \code{encodePlot} accepts raw data matrix and distance matrices for row and column margins, and from these inputs returns a formated list output with results of hcluster analysis, with options to export heatmap visualizations to working directory
#'
#' @param datmat Data frame containing the data to be clustered. Rownames of data frame must match labels of distance matrix provided.
#' @param distrow Distance matrix containing dissimilarity estimates comparing information content of each row index.
#' @param distcol Distance matrix containing dissimilarity estimates comparing information content of each column index.
#' @param clust_method_r Optional argument. Clustering method (modularity estimator) used in the base R hclust utility to cluster rows. Default is ward.D2 method.
#' @param clust_method_c Optional argument. Clustering method (modularity estimator) used in the base R hclust utility to cluster column. Default is ward.D2 method.
#' @param n_rowclusters Optional argument. Number of clusters used to encode row variable. Default is 1, wherein row order of heatmap visualization will match the dendrogram, but data will not be encoded. User may provide a range of intergers, in which case clustering and visualization routine will be executed on a grid.
#' @param n_colclusters Optional argument. Number of clusters used to encode column variable. Default is 1, wherein row order of heatmap visualization will match the dendrogram, but data will not be encoded. If 0 is provided, hclustering will not be preformed and heatmap order will match the order of observations in the dataframe. User may provide a range of intergers, in which case clustering and visualization routine will be executed on a grid.
#' @param rorderdend Optional logical arguement. Should the numerical value assigned to each encoded row value match the order that each cluster appears in the dendrogram. Default is TRUE. If FALSE, numerical value will match the order that observations appear in the data set.
#' @param corderdend Optional logical arguement. Should the numerical value assigned to each encoded column value match the order that each cluster appears in the dendrogram. Default is TRUE. If FALSE, numerical value will match the order that observations appear in the data set.
#' @param plot_title Optional argument. Title of heatmap visualization.
#' @param showplot Optional logical argument. Should heatmap visualizations be printed to screen. Default is TRUE. If FALSE heatmap plots are available in list object returned
#' @param exportplot Optional logical argument. Should heatmap visualizations be exported to jpeg. Default is FALSE
#' @param filename Optional argument. Name of file that heatmap visualization is exported to if exportplot is set to TRUE. By default image will be created in working directory. File path may be added to name to modify directory image is exported to.
#' @param imwidth Optional argument. Size of image heatmap is exported to. Default is 10 inches.
#' @param imheight Optional argument. Size of image heatmap is exported to. Default is 17 inches.
#' @param imres Optional argument. Resolution of image heatmap is exported to. Default is 300 dpi.
#' @param exportdendrogram Optional logical argument. Should dendrogram visualizations be exported to jpeg. Default is FALSE
#' @param dendheight Optional argument. Size of image dendrogram is exported to. Default is 8 inches.
#' @param dendwidth Optional argument. Size of image dendrogram is exported to. Default is 18 inches.
#' @param ... Additional options passed to pheatmap() plotting utility. If row or column margin annotations are passed, dataframes with this auxiliary data must match the row/column names of the data
#'
#' @details encodePlot encodes the information in the raw data matrix using the dissimilarity values provided by the user via a standard hclustering routine. The user is provided a discrete encodings, dendrogram information, and heatmap visualizations in a format this is compatable for downstream analysis LIT analysis and visualization utilities.
#'
#' @return Returns a list of lists. Each sublist is indexed by the parameters used to generate a given row and column encoding, and contains the following information:
#' \item{rowtree}{Hclust ouput for the row axis}
#' \item{coltree}{Hclust ouput for the column axis}
#' \item{rowclust}{Vector containing the encoding for row observations, with index names matching the label names of the dendrogram extracted from the corresponding dissimilarity matrix}
#' \item{colclust}{Vector containing the encoding for column observations, with index names matching the label names of the dendrogram extracted from the corresponding dissimilarity matrix}
#' \item{plot}{Plotting object for the heatmap visualization returned by pheatmap}
#'
#' @examples
#'
#' library(labstats)
#' data(locomotor)
#'
#' trtrats <- locomotor[locomotor$drug == 'Drug',]
#' locowide <- unstack(trtrats, dist~time)
#' rownames(locowide) <- paste('Rat', unique(trtrats$animal), sep = '')
#' names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
#'
#'
#' encodeout <- encodePlot(locowide, dist(locowide), dist(t(locowide)),
#'                     n_rowclusters = 1:5, n_colclusters = 3,
#'                     plot_title = 'Rat Locomotion Pattern Encoding')
#'
#'
#' locowide <- unstack(locomotor, dist~time)
#' rownames(locowide) <- paste('Rat', unique(locomotor$animal), sep = '')
#' names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
#' auxrow <- unique(locomotor[, c('animal', 'drug')])
#' rownames(auxrow) <- paste('Rat', auxrow$animal, sep = '')
#' auxrow <- auxrow[,'drug', drop = F]
#' names(auxrow) <- 'Treatment'
#'
#' encodeout <- encodePlot(locowide, dist(locowide), dist(t(locowide)),
#'                     n_rowclusters = 1:5, n_colclusters = 0,
#'                     plot_title = 'Rat Locomotion Pattern Encoding',
#'                     annotation_row = auxrow)
#'
#' @export



encodePlot <- function(datmat,
                       distrow,
                       distcol,
                       hclust_method_r = "ward.D2",
                       hclust_method_c = "ward.D2",
                       n_rowclusters = 1,
                       n_colclusters = 1,
                       rorderdend = TRUE,
                       corderdend = TRUE,
                       plot_title = 'Encoding Visualization',
                       showplot = TRUE,
                       exportplot = FALSE,
                       imwidth = 10,
                       imheight = 17,
                       imres = 300,
                       filename = "EncodePlotOut",
                       exportdendrogram = FALSE,
                       dendheight = 8,
                       dendwidth = 18,
                       ...){

  if(length(n_rowclusters) == 1 & length(n_colclusters) == 1){

    # create dendrograms

    hclustr <- hclust(distrow, method = hclust_method_r)
    hclustc <- hclust(distcol, method = hclust_method_c)

    # create plots

    if(n_colclusters == 0){ # if no clusters, keep order of original data
      plotout <- pheatmap::pheatmap(
        mat = datmat,
        cluster_cols = FALSE,
        cluster_rows = hclustr,
        #clustering_method = "ward.D2",
        cutree_rows = n_rowclusters ,
        #cutree_cols = n_colclusters ,
        silent = T,
        main = plot_title,
        ...)$gtable
    }else{
      plotout <- pheatmap::pheatmap(
        mat = datmat,
        cluster_cols = hclustc,
        cluster_rows = hclustr,
        #clustering_method = "ward.D2",
        cutree_rows = n_rowclusters ,
        cutree_cols = n_colclusters ,
        silent = T,
        main = plot_title,
        ...)$gtable
    }



    if(showplot){
      grid::grid.newpage()
      grid::grid.draw(plotout)
    }

    if(exportplot){
      jpeg(paste(filename,'_R',n_rowclusters,'_C',n_colclusters, '.jpeg', sep = ''),
           width = imwidth, height = imheight, units = 'in',
           res = imres)
      grid::grid.newpage()
      grid::grid.draw(plotout)
      dev.off()
    }

    # return row clusters

    if (rorderdend){
      finaltree <- hclustr
      cutout <- dendextend::cutree(as.dendrogram(finaltree),
                                   n_rowclusters,
                                   order_clusters_as_data = F)
      names(cutout) <- rownames(datmat)[dendextend::order.hclust(finaltree)]
      finalrowcluster <- cutout[match(rownames(datmat), names(cutout))]
    }
    else {
      finalrowcluster <- cutree(hclustr, n_rowclusters)
    }

    rowclustout <- finalrowcluster


    # return column clusters

    if(n_colclusters == 0){
      finalcolcluster <- NA
    }else if(corderdend) {
      finaltree <- hclustc
      cutout <- dendextend::cutree(as.dendrogram(finaltree),
                                   n_colclusters, order_clusters_as_data = F)
      names(cutout) <- colnames(datmat)[dendextend::order.hclust(finaltree)]
      finalcolcluster <- cutout[match(colnames(datmat), names(cutout))]
    }
    else {
      finalcolcluster <- cutree(hclustc, n_colclusters)
    }

    colclustout <- finalcolcluster


    listout <- list()
    listout[[paste('nrow',n_rowclusters,
                   'ncol',n_colclusters,sep ='_')]] <- list(
                     rowtree = hclustr,
                     coltree = hclustc,
                     plot = plotout,
                     rowclust = rowclustout,
                     colclust = colclustout
                   )

    # return(list(rowtree = hclustr,
    #           coltree = hclustc,
    #           plot = plotout,
    #           rowclust = rowclustout,
    #           colclust = colclustout
    #           ))

  }else{

    listout <- list()
    for(i in n_rowclusters){
      for(j in n_colclusters){

        # create dendrograms

        hclustr <- hclust(distrow, method = hclust_method_r)
        hclustc <- hclust(distcol, method = hclust_method_c)

        # creat plots

        if(j == 0){
          plotout <- pheatmap::pheatmap(
            mat = datmat,
            cluster_cols = FALSE,
            cluster_rows = hclustr,
            #clustering_method = "ward.D2",
            cutree_rows = i ,
            #cutree_cols = j ,
            #annotation_row = auxmat_tbvar,
            #annotation_colors  = auxcol_var,
            silent = T,
            main = plot_title,
            ...)$gtable
        }else{
          plotout <- pheatmap::pheatmap(
            mat = datmat,
            cluster_cols = hclustc,
            cluster_rows = hclustr,
            #clustering_method = "ward.D2",
            cutree_rows = i ,
            cutree_cols = j ,
            #annotation_row = auxmat_tbvar,
            #annotation_colors  = auxcol_var,
            silent = T,
            main = plot_title,
            ...)$gtable
        }



        if(showplot){
          grid::grid.newpage()
          grid::grid.draw(plotout)
        }

        if(exportplot){
          jpeg(paste(filename,'_R',i,'_C',j, '.jpeg', sep = ''),
               width = imwidth, height = imheight, units = 'in',
               res = imres)
          grid::grid.newpage()
          grid::grid.draw(plotout)
          dev.off()
        }

        # return row clusters

        if (rorderdend) {
          finaltree <- hclustr
          cutout <- dendextend::cutree(as.dendrogram(finaltree),
                                       i,
                                       order_clusters_as_data = F)
          names(cutout) <- rownames(datmat)[dendextend::order.hclust(finaltree)]
          finalrowcluster <- cutout[match(rownames(datmat), names(cutout))]
        }
        else {
          finalrowcluster <- cutree(hclustr, i)
        }

        rowclustout <- finalrowcluster


        # return column clusters

        if(n_colclusters == 0){
          finalcolcluster <- NA
        }else if (corderdend) {
          finaltree <- hclustc
          cutout <- dendextend::cutree(as.dendrogram(finaltree),
                                       j,
                                       order_clusters_as_data = F)
          names(cutout) <- colnames(datmat)[dendextend::order.hclust(finaltree)]
          finalcolcluster <- cutout[match(colnames(datmat), names(cutout))]
        }
        else {
          finalcolcluster <- cutree(hclustc, j)
        }

        colclustout <- finalcolcluster

        listout[[paste('nrow',i,'ncol',j,sep ='_')]] <- list(
          rowtree = hclustr,
          coltree = hclustc,
          plot = plotout,
          rowclust = rowclustout,
          colclust = colclustout
        )

      }
    }

  }

  if(exportdendrogram){
    temp <- ggdendro::ggdendrogram(hclustr,
                                   rotate = FALSE,
                                   label.size = 5)
    ggpubr::ggexport(temp,
                     width = dendwidth,
                     height = dendheight,
                     res = imres,
                     filename = paste(filename,
                                      'Dendrogram',
                                      '.pdf', sep = '_'))
  }

  return(listout)

}
