#' Calculate Ensemble-Weighted Distances
#'
#' \code{ensembleDist} computes ensemble weighted Euclidean distance measures between all pairwise combinations of rows in the obsevered data, returning a distance matrix. Penalty terms are either calculated directly the simulation array, or from pre-computed values (speed up option).
#'
#' @param po Matrix/tensor containing observed data, wherein the observational units being compared are arranged along the row axis.
#' @param pb Multidimensional array containing the ensemble of simulated data sets,  wherein the observational units being compared are arranged along the row axis, and simulations replicates are arranged on the final axis.
#' @param weightdist Optional argument. Matrix/tensor with dimensions matching Po that contains pre-computed penalty terms.
#' @param verbose Optional logical argument. Default is FALSE. If TRUE ensembleDist returns a list that contains both computed dissimilarity matrix and array containing penalty terms used.
#'
#' @details ensembleDist computes ensemble weighted Euclidean distance measures between all pairwise combinations of rows in the obsevered data array Po. The penalty terms are the ensemble variance estimates computed over B simulations along the final dimension of Pb simulation array. If verbose is set to TRUE, these penalty terms are also returned, and this matrix may be provided in subsequent calls to ensembleDist to avoid redundant calculations and thus speed up the calcultion of the dissimilarity matrix.
#'
#' Po and Pb should be identical in dimensions, with Pb having an additional axis for simulation replicates along the final axis. Observational units should always be on the first axis. If data has a time component, this should be on the second axis. Multiple features are then stored in the final axis of Po and the next-to-last axis of Pb.
#'
#' @return If verbose = FALSE, then a distance matrix is returned. If verbose = TRUE, then a list object is returned containing
#'
#' \item{dist}{The dissimilarity matrix}
#' \item{weights}{An array matching the dimensions of Po containing the ensemble variance estimates used as penalty terms to calculate the ensemble-weighted distance measurements}
#'
#'
#' @examples
#'
#' # Make Temporally Indexed Time Budget
#'
#' dat <- array(0,
#'              dim = c(25,24,5),
#'              dimnames = list(paste('Cow',1:25, sep = '_'),
#'                           paste('Hour', 0:23),
#'                           c('High Active', 'Active', 'Nonactive', 'Eating', 'Ruminating')))
#' for(i in 1:24){
#'    dat[,i,] <- t(rmultinom(25, 60, c(0.1, 0.4, 0.2, 0.05, 0.25 )))
#' }
#'
#' dat_otb <- apply(dat, c(1,3), function(x) sum(x)) # sum over time dimension
#' dat_otbp <- dat_otb/1440 # normalize by total observed minutes
#'
#'
#' # Create Ensemble of simulated overall time budgets
#'
#' simout <- simTimeBudget(dat, 100, aggregateTB = 'overall') # create ensemble of 100 simulated overall time budgets
#'
#' # Calculate Ensemble Distance
#'
#' distout <- ensembleDist(dat_otbp, simout$TBSim)
#' dist_euc <- dist(dat_otbp)
#'
#' plot(as.vector(dist_euc), as.vector(distout),
#'      xlab = 'Euclidean Distance', ylab = 'Ensemble-Weighted Distance')
#'
#'
#' # Utilize Pre-Calculated Penalty Values
#'
#' distout <- ensembleDist(dat_otbp, simout$TBSim, verbose = T)
#' as.matrix(distout$dist2)[1:5, 1:5]
#' distout2 <- ensembleDist(dat_otbp, simout$TBSim, weightdist = distout$weights , verbose = T)
#' as.matrix(distout2$dist)[1:5, 1:5]
#'
#' @export



ensembleDist <- function(po, pb, weightdist = NULL , verbose = F){
  #po = matrix/tensor of observed values
  #pb = matrix/tensor of simulated values

  B = dim(pb)[length(dim(pb))] # size of ensemble

  distout <- matrix(0, nrow = nrow(po), ncol = nrow(po))
  rownames(distout) <- rownames(po)
  colnames(distout) <- rownames(po)

  if(length(dim(po)) == 2){

    if(is.null(weightdist)){

      #   weightdist <- po
      #   weightdist[,] <- NA
      #
      #   for(i in 1:nrow(po)){
      #     for(j in 1:ncol(po)){
      #       weightdist[i,j] <- sum((po[i,j] - pb[i,j, ])^2)/B
      #     }
      #   }
      #

      weightdist <- apply(pb, c(1,2), var)

    }



    # start_time <- Sys.time()
    # for(i in 1:nrow(po)){
    #   for(j in i:nrow(po)){
    #
    #     distout[i,j] <- sum((po[i,] - po[j,])^2/
    #                           (weightdist[i,] + weightdist[j,])
    #                         )
    #     distout[j,i] <- distout[i,j]
    #
    #   }
    # }
    # end_time <- Sys.time()
    # end_time - start_time
    # dist1 <- distout

    distout <- matrix(0, nrow = nrow(po), ncol = nrow(po))
    rownames(distout) <- rownames(po)
    colnames(distout) <- rownames(po)

    #start_time <- Sys.time()
    for(j in 1:ncol(po)){
      distout <- distout +
        as.matrix(Rfast::Dist(matrix(po[,j], ncol = 1), square = T))/
        outer(weightdist[,j], weightdist[,j], FUN = '+')
    }
    # end_time <- Sys.time()
    # end_time - start_time
    # dist2 <- distout


  }else if(length(dim(po)) == 3){

    if(is.null(weightdist)){

      weightdist <- po
      weightdist[,,] <- NA

      for(i in 1:nrow(po)){
        for(j in 1:ncol(po)){
          for(k in 1:dim(po)[3]){
            weightdist[i,j,k] <- sum((po[i,j,k] - pb[i,j,k, ])^2)/B
          }
        }
      }

    }

    for(i in 1:nrow(po)){
      for(j in i:nrow(po)){

        distout[i,j] <- sum((po[i,,] - po[j,,])^2/
                              (weightdist[i,,] + weightdist[j,,])
        )
        distout[j,i] <- distout[i,j]

      }
    }

  }else{
    stop('Error: Incorrect number if dimensions')
  }


  if(verbose){
    return(list(dist = as.dist(distout), weights = weightdist))
  }else{
    return(as.dist(distout))
  }

}
