#' Create Discrete Encoding
#'
#' \code{getEncoding} takes a data set and returns a discrete encoding generated via a heirarchical clustering stratedgy specified by the user
#'
#' @param x Data matrix or data frame, with observational units arranges in the rows and features on the columns
#' @param nclust Optional argument. Number of clusters to extract to form discrete encoding. Default is 2 for binarization.
#' @param dist_method Optional argument. Dissimilarity estimate used to encode data. Options are 'Euclidean', 'KLD, or 'Ensemble'. Default is Euclidean.
#' @param hclust_method Optional argument. Agglomeration method employed to create dendrogram. Default is 'ward.D2'. See hclust for more options.
#' @param simdata Optional argument. Simiulated data set used to calculate ensemble weighted distances. See ensembleDist for more details.
#' @param weightdist Optional argument. Precomputed weight values that may be used to calculate ensemble weighted distances. See ensembleDist for more details.
#' @param returnTree Optional logical argument. Default is FALSE. Should dendrogram used to create encoding also be returned
#'
#' @return If returnTree iss FALSE, only a vector with encoded values is returned. If returnTree is TRUE, a named list is returned that also containing:
#'
#' \item{cluster}{Vector containing discrete encoding that matches row index of the original data}
#' \item{hctree}{Dendrogram object from which the discrete encoding is derived}
#'
#' @examples
#'
#' library(labstats)
#' data(locomotor)
#'
#' locowide <- unstack(locomotor, dist~time)
#' rownames(locowide) <- paste('Rat', unique(locomotor$animal), sep = '')
#' names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
#'
#' discout <- getEncoding(locowide, 4, returnTree = T)
#' discout$cluster
#' plot(discout$hctree)
#'
#' @export

getEncoding <- function(x,
                          nclust = 2,
                          dist_method = 'Euclidean',
                          hclust_method = 'ward.D2',
                          simdata = NULL,
                          weightdist = NULL,
                          returnTree = FALSE){

  ## Claculate Dissimilarity Matrix ##

  if(dist_method == 'Euclidean'){
    distmat <- dist(x)
  }
  if(dist_method == 'KLD'){
    distmat <- KLDdist(x, runfast = T)
    #distmat <- as.dist(Rfast::Dist(x, method='kullback_leibler'))
  }
  if(dist_method == 'Ensemble'){
    distmat <- ensembledist(x, simdata, weightdist = weightdist)
    wout <- ensembledist(x, simdata, weightdist = weightdist)
    #distmat2 <- j
  }


  ## Create Binary Code ##

  hout <- hclust(distmat, method = hclust_method)
  cout <- cutree(hout, nclust)

  if(returnTree){
    return(list(cluster = cout, hctree = hout))
  }else{
    return(cout)
  }

}
