#' Generate a tube plot by grouping variable
#'
#'  \code{groupTubeplot} calculates a summary statistic (mean, median, etc) at each unique time point across groups of observational units and uses these values to generate a \code{\link{tubeplot}} for each respective group
#'
#' @param cyc A numeric vector containing cyclical indexing variable for each observation
#' @param long A numeric vector containing longitudinal temporal index for each observation
#' @param dat A data frame with a row for each unique timepoint observed and column for each observational unit (ie - animal)
#' @param g Vector containing the group assignments for the corresponding columns in dat
#' @param func Optional arguement. String specifying base R operation used to summarize data at each time point across each group. Defualt is median.
#' @param na.rm Optional arguement. Logical specifying if NA values should be dropped in calculating summarizing statistics for each group. Default is to drop NA values.
#' @param subsethour Optional arguement. If a subset of cyclical time indices are provided, data points not selected will be set to grey
#' @param showplot Optional argument. Should plot be printed to screen. Default is TRUE
#' @param export_html Optional argument. Should interactive html file be saved to current working directory. Default is FALSE
#' @param plot_title Optional argument. Title added to plot. Defaults to no title.
#' @param filename Optional argument. Name of html file exported to current working directory. Do not include file extension. Default is 'TubePlot'
#' @param makediffplot Optional arguement. Calculate the difference between each pairwise observations of groups at each time point and plot the difference. Default is FALSE
#' @param returnstats Optional arguement. If TRUE, returns a dataframe with summary statistics computed at each time point for all groups. Default is FALSE
#' @param ... Additional arguements passed into tubeplot()
#'
#' @return Returns a 3D tubeplot for each group of observational units. Information is compressed at each time point across individuals within the group using a summary statistic, usually a measure of central tendancy such as median or mean. These summary values are then used to determine the value (color) of each marker comprising the tube plot. the makediffplot option will also allow users to return tubeplots representing the contrasts between summary statistics at each time point for all pairwise combination of groups. The returnstats option allows users to return the data frame of summary statistics at each time points for all groups (useful for checking code).
#'
#' @examples
#' v_long <- rep(1:40, each = 24)
#' v_cyc <- rep(1:24, times = 40)
#' obsdat <- data.frame(Cow1 = v_cyc + v_long,
#'                      Cow2 = v_cyc + v_long,
#'                      Cow3 = v_cyc + 2*v_long,
#'                      Cow4 = v_cyc + 2*v_long)
#' cowgroups <- c(1,1,2,2)
#' groupTubeplot(v_cyc, v_long, obsdat, cowgroups)
#'
#' @export



groupTubeplot <- function(cyc, long, dat, g, func = 'median', na.rm = T, subsethour = c(), showplot = T, plot_title = 'TubePlot', export_html = F, filename = 'TubePlot', makediffplot = F, returnstats = F, ...){

  #require('plotly')

  tempobs <- matrix(NA, ncol = length(unique(g)), nrow = nrow(dat))

  for(tempg in 1:length(unique(g))){

    tempsubdat <- dat[,g==tempg]
    tempobs[ ,tempg] <- apply(tempsubdat, 1, func, na.rm = na.rm)
    #tempobs[ ,tempg] <- do.call(apply, list(X = tempsubdat, MARGIN = 1, FUN = func, ... = funcoptions))
  }

  if(length(subsethour)>0){
    tempobs[which(!cyc%in%subsethour), ] <- NA
  }


  # standardize colorscale across groups

  mycolormin = min(tempobs, na.rm = T) # standardize colorscale across groups
  mycolormax = max(tempobs, na.rm = T)
  temp <- min(which(apply(tempobs, 1, function(x) sum(is.na(x))) == 0)) # make sure markers are present for all groups to overplot

  # put these hidden values at top of the data frame so they will get overplotted
  tempobs <- rbind(rep(mycolormin, ncol(tempobs)), tempobs)
  tempobs <- rbind(rep(mycolormax, ncol(tempobs)), tempobs)
  cyc <- c(cyc[temp], cyc[temp], cyc)
  long <- c(long[temp], long[temp], long)



  # c <- c(c,13)
  # r <- c(r,max(r)+1)
  # tempobs <- rbind(tempobs, rep(mycolormax, ncol(tempobs)))
  #tempobs <- rbind(tempobs, rep(NA, ncol(tempobs)))
  #temp <- c(tempobs[,1], mycolormax, NA)
#
#   # fix max
#   tempobs[cyc==min(cyc, na.rm = T) & long == min(long, na.rm = T),] <- rep(mycolormax, ncol(tempobs))
#   cyc <- c(cyc,13)
#   r <- c(long,min(long))
#   tempobs <- rbind(tempobs, rep(NA, ncol(tempobs)))
#
#   #fix min
#   tempobs[c==13 & r == max(r, na.rm = T),] <- rep(mycolormin, ncol(tempobs))
#   c <- c(c,13)
#   r <- c(r,max(r))
#   tempobs <- rbind(tempobs, rep(NA, ncol(tempobs)))

  outplot <- list()

  for(tempg in 1:length(unique(g))){

    outplot[[length(outplot) + 1]] <- tubeplot(cyc, long, tempobs[,tempg],
                                             showplot = showplot,
                                             export_html = export_html,
                                             filename = paste(filename, '_Group', tempg, sep=''),
                                             plot_title = paste(plot_title, ': Group ', tempg, sep=''),
                                             ...)

    if(makediffplot){
      if(tempg<length(unique(g))){
        for(contg in (tempg + 1):length(unique(g))){

          conts = tempobs[,tempg] - tempobs[,contg]
          outplot[[length(outplot) + 1]] <- tubeplot(cyc, long, conts,
                                                   showplot = showplot,
                                                   export_html = export_html,
                                                   filename = paste(filename, '_ContrastG', tempg, 'vG', contg, sep=''),
                                                   plot_title = paste(plot_title, ': Contrast Group ', tempg, ' vs Group ', contg, sep=''),
                                                   ...)

        }
      }
    }


  }

  if(showplot){
    if(returnstats){

      datout <- cbind(data.frame(Cyclical_Index = cyc,
                           Longitudinal_Index = long),
                      as.data.frame(tempobs))
      names(datout)[-c(1,2)] <- paste('Group_', unique(g), sep = '')

      return(list(plots = outplot, summarydata = datout))
    }else{
      return(outplot)
    }

  }

}
