#' Normalize Dissimilarity Matrix
#'
#' \code{normTreeDist} accepts the output from cutreeEnsemble, and returns a dissimilarity matrix that has been pruned using the validated portions of the dendrogram
#'
#' @param cutout Output from cutreeEnsemble.
#' @param normalize. Optional logical argument. Default is TRUE. Should the distance matrix values be normalized by the length of the longest validated branch.
#'
#' @details normTreeDist accepts the output from the cutreeEnsemble utility. It then reconstructs the dissimilarity matrix such that the dissimilarity value between any two pairs of observational units is the longest distance between two validated branches. The dissimilarity between pairs of observational units that have been found to have been deemed to be indistinguishable relative the resolution of the resolution of the data set and pruned into the same branch are assigned dissimilarity values of 0. By default the pruned dissimilarity matrix is normalized relative the length of the longest validated branch so that the longest dissimilarity matrix that may be assigned between any two pairs of observations will be 1. If this option is set to FALSE dissimilarity values will be extracted directly from the observed dissimilarity matrix.
#'
#' If the output from normTreeDist is passed through an hclustering utility using agglormeration method 'single', the resulting dendrogram will match that of the original data for all stochastically validated branches, with any insignificant branches removed and the observational units randomly ordered within the nearest validated branch to help better visualize the heterogenity within each branch.
#'
#' @return A distance matrix
#'
#' @export

normTreeDist <- function(cutout, normalize = T){


  roworder <- names(cutout$FinalEncoding)
  distmat <- as.matrix(cophenetic(cutout$Dendrogram))

  for(i in 1:nrow(distmat)){
    tempcow <- rownames(distmat)[i]
    temp <-names(cutout$FinalEncoding)[cutout$FinalEncoding[tempcow]
                                       == cutout$FinalEncoding]
    distmat[i, temp] <- 0

  }

  distmat <- distmat[roworder, roworder] # reorder dist mat to match order of orignal data

  if(normalize){
    distmat <- distmat/max(distmat)
  }

  return(as.dist(distmat))

}
