#' Conditionally Permute Edge Weights
#'
#' \code{permuteEdgeWeights} performs a stochastic permutation of edge weight values within a network adjacency matrix, excluding NA edgeweights. Users may provide an additional matrix, conformable with the adjacency, that directs the permutation of edge weights within pre-defined subgroups
#'
#' @param adj An adjacency matrix. Must be  symmetric and formatted as a matrix.
#' @param cond Optional argument. A symmetric matrix with dimensions equivalent to the corresponding adjacency matrix. Values in this matrix indicate which permutation subgroup each edge in the corresponding index position in the adjacency matrix should be permuted within. Any edges marked as NA in the conj matrix will not be permuted, and by default all NA values in adj matrix will be set to NA in conj
#' @return Returns an adjacency matrix with permuted edge weights. Note that values along diagonal will not be permuted and will be returned as 0's
#'
#' @examples
#'
#' % permute edges with the same the even/odd modularity
#'
#' adj <-  matrix(1:100, nrow = 10)
#' adj[upper.tri(adj, diag = F)] <- 0
#' adj <- t(adj) + adj
#' adj
#'
#' conj <- adj %%  2
#' conj
#'
#' permuteEdgeWeights(adj, conj)
#'
#'
#' @export

permuteEdgeWeights <- function(adj, cond = NULL){

  ### Setup ###

  if(!is.matrix(adj)){stop('Error: Adjacency matrix must be formated as a matrix')}
  if(!isSymmetric(adj)){stop('Error: Adjacency matrix must be symmetric')}


  permout <- matrix(NA, nrow = nrow(adj), ncol = ncol(adj)) # initialize output matrix
  rownames(permout) <- rownames(adj) # transfer row/column names from input adjacency to simulated adjacency
  colnames(permout)<- colnames(adj)

  if(is.null(cond)){ # set up to perform full edge permutation with no conditional values
    cond <- adj
    cond[!is.na(cond)] <- 1 # initialize conditional matrix to have only one group for all non-NA edges
    cond[upper.tri(cond, diag = F)] <- NA # make the conditional matrix lower triangular
  }else{ # check that the provided conditional matrix has the correct format
    if(sum(dim(cond) != dim(adj)) != 0){stop('Error: Conditional matrix provided must have same dimensions as the adjacency')}
    if(!is.matrix(cond)){stop('Error: Conditional matrix must be formated as a matrix')}
    cond[is.na(adj)] <- NA # set any edges in adj with an NA weight to NA in conj
    cond[upper.tri(cond, diag = T)] <- NA # make the conditional matrix lower triangular
  }

  ### Perform Permutation ###

  for(myg in names(table(cond))){ # parse through groups in cond, permute edge weights in adj assigned to a given group

    permout[cond == myg & !is.na(cond)] <- sample(adj[cond == myg & !is.na(cond)])

  }


  ### Return Permutation ###

  if(sum(diag(adj) != 0, na.rm = T) == 0){
    diag(permout) <- 0 # set diagonal back to zero if adj diagonal was zero
  }else{
    # if adj diagonal has nonzero values, the diagonal will be returned with NA values
  }

  permout[upper.tri(permout, diag = F)] <- t(permout)[upper.tri(permout, diag = F)] # make output symmetric
  return(permout)

}
