#' Permute Network Tensor
#'
#' \code{permuteNetworkTensor} independently permutes either the node ID's or edge weights for each adjacency submatrix within the network tensor, allowing for the permutation to be conditioned on auxiliary data.
#'
#' @param tensordat Network tensor array. Each temporally independent adjacency matrix should be defined along the first two dimensions of the array, with temporal dimension as the third axis of the array.
#' @param B Number of permutations to generate. Default value is 100.
#' @param permutenodes Optional argument. Logical argument indicating if a nodal ID permution should be implemented (default) or an edge permutation.
#' @param auxdat Optional argument. Data used to perform a conditioned permutation across subgroups for each indepdendent auxiliary matrix. See details.
#' @param tensorout Optional argument. Logical argument indicating if the full network tensor should be returned after permutation (default) or just the weighted adjacency matrix aggregated over the time domain after the permutation is performed.
#'
#' @return List of length B containing either the permuted network tensor arrays or the weighted adjacency matrices
#'
#' @details If a conditional nodal ID permutation is performed, auxdat should be formatted as a data frame. If the conditioning variable is uniform across adjacency matrices, then data frame should have a single column, with rows matching those of the adjacency submatrices in dimension and rownames. If the conditioning variable changes across the time axis, then the number of columns in the auxdat data frame should match the length of the tensor arrays third time dimension, and the names of the auxdat columns must match the names of the third dimension of the tensor array.
#'
#' If a conditional edge weight permutation is performed, auxdat should be formatted as either an array or a matrix. If the conditioning variable is uniform across adjacency matrices, then a matrix identifcal in size and row/column names to the first two dimensions of the network tensor can be provided. If the conditioning variable changes across the time axis, then an array with equivalent size and dimname specs to that of the network tensor array should be provided.
#'
#' For more information on how nodal and edge permutations are performed, including how missing observations are handled, see {permuteEdgeWeights} and {permuteNodeID}
#'
#' @export



permuteNetworkTensor <- function(tensordat, B = 100, permutenodes = T, auxdat = NULL, tensorout = TRUE){


  # check input

  if(!is.array(tensordat)){stop('Error: Network tensor must be formatted as array')}
  if(B%%1 != 0 | B <= 0){stop('Error: B must be a positive integer')}


  # Simulation

  outdat <- list()
  for(b in 1:B){

    if(permutenodes){

      ### check input ###
      if(!is.null(auxdat)){
        if( !is.data.frame(auxdat) ){stop('Error: Auxdat must be formatted as data frame')}
        if(ncol(auxdat) > 1){
          if(ncol(auxdat) != dim(tensordat)[3]){stop('Error: number of columns in auxdat must either be 1 or match length of third dimension of network tensor array')}
          if(sum(names(auxdat) != dimnames(tensordat)[[3]]) > 0){stop('Error: Names of third axis of network tensor must match column names of auxdat')}
        }
      }

      ### perform nodal permutation ###

      if(is.null(auxdat)){
        outdat[[b]]<- array(apply(tensordat, 3, function(x)
          permuteNodeID(x)),
          dim = dim(tensordat),
          dimnames = dimnames(tensordat))
      }else{
        outten <- array(NA,
                        dim = dim(tensordat),
                        dimnames = dimnames(tensordat))
        for(i in 1:dim(tensordat)[3]){
          outten[,,i] <- permuteNodeID(tensordat[,,i], auxdat, i)
        }
        outdat[[b]] <- outten
      }

    }else if(permutenodes == F){

      ## check input ##

      if(!is.null(auxdat)){

        if(!is.array(auxdat)){stop('Error: Auxdat must be either a matrix or an array')}
        if(!is.matrix(auxdat)){
          if(dim(tensordat)[3] != dim(auxdat)[3]){stop('Error: Dimension of temporal dimension of auxdat and network tensor array must match')}
          if(sum(dimnames(tensordat)[[3]] != dimnames(auxdat)[[3]]) > 0){stop('Error: Dim names of network tensor must match dimnames of auxdat array along the temporal dimension')}
        }

      }

      ## perform permutation ##

      if(is.null(auxdat)){
        outdat[[b]]<- array(apply(tensordat, 3, function(x)
          permuteEdgeWeights(x)),
          dim = dim(tensordat),
          dimnames = dimnames(tensordat))
      }else{
        outten <- array(NA,
                        dim = dim(tensordat),
                        dimnames = dimnames(tensordat))
        for(i in 1:dim(tensordat)[3]){
          if(length(dim(auxdat)) == 2){
            outten[,,i] <- permuteEdgeWeights(tensordat[,,i], auxdat) # permute with static conditional variables
          }else{
            outten[,,i] <- permuteEdgeWeights(tensordat[,,i], auxdat[,,i]) # permute with dynamic conditional variables
          }


        }
        outdat[[b]] <- outten
      }

    }else{
      stop('Error: permutenodes option must be logical T/F')
    }

  }



  # Format output

  if(tensorout){
    return(outdat) # return list of simulated tensors
  }else{
    adjout <- lapply(outdat, function(x)
      aperm(apply(x, c(1,2), function(y)
        sum(y, na.rm = T)),
        dim = dim(x))
    )
    return(adjout)
  }


}





