#' Conditionally Permute Node IDs
#'
#' \code{permuteNodeID} performs a stochastic permutation of node ID's within a network adjacency matrix, skipping nodes with only NA edge weights. Users may provide auxiliary data that directs the permutation of node IDs within pre-defined subgroups
#'
#' @param dat An adjacency matrix.
#' @param auxdat Optional argument. A data frame containing information on which to condition the node permutation into subgroups. Rownames must match those of the auxiliary matrix provided.
#' @param auxdatindex Optional argument. Integer indicating the column index containing the data to condition the permutation upon.
#'
#' @details Node ID permutations will only be performed on nodes with at least one recorded non-NA value. If auxiliary data is provided on which to condition the node ID permutation, any sows with an NA auxiliary value will not be permuted.
#'
#' @return Returns an adjacency matrix with permuted node IDs
#'
#' @examples
#'
#' adj <- matrix(rep(1:10, each = 10), nrow = 10)
#' adj[upper.tri(adj, diag = T)] <- 0
#' adj <- adj + t(adj)
#' adj[c(2,5), ] <- NA
#' adj[, c(2,5)] <- NA
#' rownames(adj) <- 1:10
#' colnames(adj) <- 1:10
#'
#' auxdat <- data.frame(EvenOdd = rep(1:2, 5))
#' rownames(auxdat) = rownames(adj)
#' colnames(auxdat) = colnames(adj)
#'
#' permuteNodeID(adj)
#' permuteNodeID(adj, auxdat)
#'
#' @export

permuteNodeID <- function(dat, auxdat = NULL, auxdatindex = NULL){

  # check dat
  if(!isSymmetric(dat)){stop('Error: Adjacency matrix must be symmetric')}
  if(is.null(rownames(dat)) | is.null(colnames(dat))){stop('Error: Adjacency matrix must have names rows and columns')}
  if(sum(!rownames(dat) == colnames(dat))>0){stop('Error: row names must match column names for association matrix')}

  # check auxdat
  if(!is.null(auxdat)){
    if(!is.data.frame(auxdat)){
      stop('Error: Auxdat must be formatted as data frame')
    }
    if(nrow(auxdat) != nrow(dat)){
      stop('Error: Length of auxiliary vector must match dimensions of adjacency matrix')
    }
    if(sum(rownames(auxdat)!= rownames(dat))>0){
      stop('Error: Order of auxdat must match order of adjacency matrix')
    }


    if(ncol(auxdat) > 1){
      auxdat <- as.factor(auxdat[ ,auxdatindex])
    }else{
      auxdat <- as.factor(auxdat[ ,1])
    }

  }



  if(is.null(auxdat)){

    obstemp <- which( apply(!is.na(dat), 1, sum) > 0 ) # indices of sows observed in that association matrix (not all obs values NA)
    temp <- rownames(dat)
    temp[obstemp] <- sample(temp[obstemp]) # permute indices of present sows
    tempmat2 <- dat[temp, temp]
    rownames(tempmat2) <- rownames(dat)
    colnames(tempmat2) <- colnames(dat)

  }else{
    tempmat2 <- dat
    for(i in levels(as.factor(auxdat))){
      temp <- rownames(dat)
      obstemp <- which(auxdat == i &
                         apply(!is.na(dat), 1, sum) > 0) # indices within category to be permuted
      temp[obstemp] <- sample(temp[obstemp]) #permute indices of flagged sows
      tempmat2 <- tempmat2[temp, temp]
      rownames(tempmat2) <- rownames(dat)
      colnames(tempmat2) <- colnames(dat)
    }


  }

  return(tempmat2)

}


