#' Generate a possibly-gapped histogram
#'
#'  \code{pghist} creates a possibly graphed histogram using hclust, optionally returning a distcretized encoding of the continuous input and additional visualizations
#'
#' @param obs Numeric vector containing the observations of the continous variable to be encoded.
#' @param nbins Number of histogram bins (ie - number of categories used to encode data).
#' @param freq Optional argument. If TRUE (default) histogram displays frequency of obs in each bin. If FALSE displays density of obs in each bins.
#' @param clustermethod Optional argument. Method used to create clusters (see hclust for options). Default is Ward D2.
#' @param xlab Optional argument. Label for x-axis of histogram.
#' @param main Optional argument. Title of histogram.
#' @param col Optional argument. Color of histogram bins
#' @param showDendrogram Optional argument. Return plot of dendorgram returned by hclust. Default is FALSE.
#' @param showCDFPlot Optional argument. Return plot showing discretized approximation of continuous empirical CDF. Default is FALSE.
#' @param returnCluters Optional argument. Return list object containing cluster assignments and details

#' @return Optionally returns a list containing
#' \item{ClusterSpecs}{Matrix with Min and Max values for each bin}
#' \item{Encoding}{Vector containing bin assignments (intergers) for each observation in the corresponding obs input vector}
#'
#'
#' @examples
#' library(LIT)
#' set.seed(61916)
#' obs <- c(rnorm(30, 0, 2), rnorm(30, 10, 1), rnorm(30, 15, 1))
#' hist(obs, breaks = 12)
#' mycode <- pghist(obs, 12, returnCluters = TRUE)
#' data.frame(Obs = obs, Encoding = mycode$Encoding)


pghist <- function(obs, nbins, freq = TRUE, clustermethod = 'ward.D2', xlab = 'Observed Data', main = 'Possibly Gapped Histogram', col = 'slateblue', showDendrogram = FALSE, showCDFPlot = FALSE, returnCluters = FALSE){

  require(dendextend)

  names(obs) <- obs
  hout <- hclust(dist(obs), method = clustermethod)

  if(showDendrogram){
    plot(hout) #, labels = F
    dendextend::rect.dendrogram(as.dendrogram(hout), k = nbins)
  }


  clustout <- cutree(hout, nbins)
  minmaxout <- cbind(tapply(obs, clustout, min),
                     tapply(obs, clustout, max))
  clustorder <- order(minmaxout[, 1])
  minmaxout <- minmaxout[clustorder, ]

  clustdensity <- table(clustout)[clustorder] / length(obs)
  cumdensity <- cumsum(clustdensity)

  clustfreq <- table(clustout)[clustorder]
  cumfreq <- cumsum(clustfreq)

  if(freq){

    plot(NULL,
         xlim = c(min(obs),max(obs)),
         ylim = c(0, max(clustfreq)),
         ylab = "frequency",
         xlab = xlab,
         main = main)
    for(j in 1:nbins){
      rect(minmaxout[j,1], 0, minmaxout[j,2], clustfreq[j], col = col)
    }

  }else{

    plot(NULL,
         xlim = c(min(obs),max(obs)),
         ylim = c(0, max(clustdensity)),
         ylab = 'density',
         xlab = xlab,
         main = main)
    for(j in 1:nbins){
      rect(minmaxout[j,1], 0, minmaxout[j,2], clustdensity[j], col = col)
    }

  }


  if(showCDFPlot){

    plot(ecdf(obs), col="grey",
         cex=0.2,
         main = 'Empirical CDF',
         xlab = 'Observed Data')
    segments(minmaxout[1,1], 0, minmaxout[1,2], cumdensity[1], col="red")
    for( j in 1:(nbins-1)){
      segments(minmaxout[(j+1),1], cumdensity[j],
               minmaxout[(j+1),2], cumdensity[(j+1)], col="red" )
    }

  }



  if(returnCluters){

    colnames(minmaxout) <- c('min','max')
    rownames(minmaxout) <- 1:nrow(minmaxout)

    names(obs) <- 1:length(obs)
    cutout <- cutree(hclust(dist(obs[order(obs)]),
                            method = clustermethod),
                     nbins)
    codeout  <- cutout[match(names(obs), names(cutout))]


    return(list(ClusterSpecs = minmaxout, Encoding = codeout))

  }

}
