#' Calculate Relative Entropy
#'
#' \code{relativeEntropy} computes the relative entropy between two distributions with the same support as in MacKay (2003)
#'
#' @param p Vector of counts/frequencies for an observed distribtuion ('p' in MacKay 2003)
#' @param q Vector of counts/frequencies for a distribtuion that is serving as a standard of comparison ('q' in MacKay 2003)
#' @param isFreq Optional logical argument. Default is TRUE, indicating p and q are provided as frequencies. User may provide vector of counts if isFreq is FALSE.
#' @param logbase Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).
#' @param DirichletCorrection Optional logical argument. Default is FALSE. Should a Dirichlets correction be applied to frequency vectors before relative entropy is calculated. Allows for entropy to be calculated if q may contain 0 due to under-sampling
#' @param Np Optional argument. If isFreq is TRUE, then a total count must be provided for p in order to compute a Dirichlets correction
#' @param Nq Optional argument. If isFreq is TRUE, then a total count must be provided for q in order to compute a Dirichlets correction
#'
#'
#' @details relativeEntropy calculates the relative entropy between two discrete distributions. The two distributions may be provided either as frequency (normalized to sum to 1) or as count vectors, but must have the same domain of support (ie - be the same length)
#'
#'
#' @examples
#'
#' pcount <- c(10, 20, 30, 0)
#' p <- pcount/sum(pcount)
#'
#' qcount <- pcount  + 1
#' q1 <- qcount/sum(qcount)
#' q2 <- rev(pcount + 1)/sum(pcount  + 1)
#'
#' relativeEntropy(p, q1) # calc relative entropy using frequencies
#' relativeEntropy(pcount, qcount, isFreq = F)# calc relative entropy using counts
#' relativeEntropy(p, q2) # q2 has a different underlying pattern, so has higher relative entropy than when q1 is used as reference
#'
#' relativeEntropy(q1, p) # because p contrains a zero-count, it will return an INF when used as a reference dist
#' relativeEntropy(qcount, pcount, isFreq = F, DirichletCorrection = T) # applying a Dirichlets correction to get a relative entropy estimate in spite of the zero-count in the reference distribution
#' relativeEntropy(q1, p, DirichletCorrection = T, Np = sum(qcount), Nq = sum(pcount)) # applying a Dirichlets correction to get a relative entropy estimate in spite of the zero-count in the reference distribution
#'
#'
#' @export


relativeEntropy <- function(p, q, isFreq = TRUE, logbase = 2, DirichletCorrection = FALSE, Np = NULL, Nq = NULL){

  ### check format of dist objects ###

  if(!is.vector(p)){
    if(dim(!1%in%dim(p))){
      stop('Error: Distributions must be provided as vectors')
    }
  }

  if(!is.vector(q)){
    if(dim(!1%in%dim(q))){
      stop('Error: Distributions must be provided as vectors')
    }
  }


  if(length(p) != length(q)){
    stop('Error: Distributions must have the same domain of support (ie - vectors should be the same length)')
  }

  ### check normalization of distributions ###

  if(!isFreq){ # normalize dist vectors if provided as counts

    Np <- sum(p)
    p <- p/Np # normalize p

    Nq <- sum(q)
    q <- q/Nq # normalize q

  }


  if(sum(p) != 1){ # check normalization of p
    stop('Error: Distribution p does not sum to 1')
  }
  if(sum(q) != 1){ # check normalization of q
    stop('Error: Distributions q does not sum to 1')
  }


  ### Apply Corrections ###

  if(DirichletCorrection){
    if(is.null(Np) | is.null(Nq) ){stop('Error: Total counts (Np/Nq) must be provided to calculate dirichlet correction if isFreq is TRUE')}

    countp <- Np * p
    countq <- Nq * q

    p <- (countp + 1)/sum(countp + 1)
    q <- (countq + 1)/sum(countq + 1)

  }


  ### calculate relative entropy ###

  return( sum(p * log(p/q, base = logbase), na.rm = T) )

}
