#' Simulate Time Budget Data
#'
#' \code{simTimeBudget} returns a multidimensional array containing an ensemble of simulated data sets that mimicks the variability in observed time budget data
#'
#' @param dat Data matrix/tensor that contains the observed time budget data. Observational units should be on the first axis. If the data is generated with repeated samples, the time dimension should be indexed on the second axis, and the multivariate features should be indexed on the final axis.
#' @param nsim Optional argument. Number of simulations. Defualt is 100.
#' @param simMethod Optional argument. Parametric simulation stratedgy used to simulate noise in time budget data. Optional are 'MN' (multinomial), 'MVB' (multivariate beta with Dirichlets correction), or 'DUAL' (mutinomial redrawn from multivariate beta sample with Dirichlets correction). Default is 'DUAL'
#' @param aggregateTB Optional argument. By default, time index of the simulated data will match that of the raw data. If set to 'overall' temporal axis will be compressed to produce an ensemble of overall time budgets. If 'byvar' user may pass an auxiliary time index over which to conditionally compress the temporal axis.
#' @param agVar Optional argument. Auxiliary time index passed by the user over which to conditionally compress the time axis in the ensemble time budgets if aggregateTB = 'byvar'
#' @param NTempSubsamp Optional argument. If data is temporally indexed, number of subsamples to retain in calculating ensemble values. By default no subsampling will be applied.
#' @param subsampVar Optional argument. User may pass auxiliary time index used to conditionally subsample the data. By default, the subsampling utility will subsample unconditionally across the time axis of the observed data without replacement.
#' @param verbose Optional arguement. Default is TRUE. If time axis is aggregated, verbose set to TRUE will return simulation result both before and after aggregation over the time axis, while if verbose is set to FALSE data will the final aggregated ensemble simulations are returned.
#'
#' @details simTimeBudget produces an ensemble of semi-parametric simulations that strive to emulate the error structure in observed time budget data absent any assumption of the underlying structural features.
#'
#' If the observed count data is stored as a aggregate overall time budget, it may be passed as a 2D matrix with observational units (usually animals) on the row axis and features (usually behaviors) on the column axis. If observed count data is sampled as a repeated measure, data is passed as a 3D array (tensor) with time on the second axis and behavioral features on the last axis.
#'
#' In general, data should be simulated at the finest temporal granularity provided by the sensor in order to better emulate the temporal heterogenity of the model system. In many cases, data may be analyzed either as an overall time budget or at a courser time index, such as daily time budgets. simTimeBudget allows users to return an ensemble of simulations at the desired temporal level using the 'aggregateTB' and 'agVar' arguments.
#'
#' simTimeBudget will return data in both counts and normalized proportion formats. If data is aggregated over the time axis, by default data will be returned for both the raw and temporally aggregated ensemble. If verbose is FALSE, only the temporally aggregated ensemble is returned, which can reduce memory pressure with large files.
#'
#' When temporal aggregation is applied over the time index, it is also possible to emulate uncertainty in the data due to temporal stationarity/plasticity of the data over time by subsampling the data prior to aggregation (conceptually this is similar to jacknife estimates of variance). By default time points are subsampled unconditionally without replacement. For data sets where time indices are heirachically nested, it may make sense to subsample by an auxiliary variable. For example, the user may which to subsample across days to ensure that all hours within the 24-hour circadium rhythem are given equal representation in the final ensemble of simulations. Generally, the smaller the number of subsamples selected, the higher the ensemble variance will become due to increasing sampling error.
#'
#' @return A named list containing
#'
#' \item{DataSim}{Multidimensional array containing simulated count data, wherein the simulation replicate index is stored on the final axis}
#' \item{TBSim}{Multidimensional array containing simulated normalized time budget proportions, wherein the simulation replicate index is stored on the final axis}
#' \item{FullSimResult}{Returned if aggregateTB is not NULL and verbose = TRUE. A list containing a DataSim and TBSim object as above containing the simulation results prior to aggregation over the time axis}
#' \item{SimSpecs}{A named list containing argument specifications needed to replicate a simulation result}
#'
#'
#' @examples
#'
#' # Overall time budget
#'
#' dat <- t(rmultinom(25, 60, c(0.1, 0.4, 0.2, 0.05, 0.25 )))
#' rownames(dat) <- paste('Cow',1:25, sep = '_')
#' colnames(dat) <- c('High Active', 'Active', 'Nonactive', 'Eating', 'Ruminating')
#'
#' simout <- simTimeBudget(dat, 1)
#'
#' # Temporally Indexed Time Budget
#'
#' dat <- array(0,
#'              dim = c(25,24,5),
#'              dimnames = list(paste('Cow',1:25, sep = '_'),
#'                           paste('Hour', 0:23),
#'                           c('High Active', 'Active', 'Nonactive', 'Eating', 'Ruminating')))
#' for(i in 1:24){
#'    dat[,i,] <- t(rmultinom(25, 60, c(0.1, 0.4, 0.2, 0.05, 0.25 )))
#' }
#'
#' simout <- simTimeBudget(dat, 1)  # ensemble will match dimensions of original data
#' simout <- simTimeBudget(dat, 1, aggregateTB = 'overall') # ensemble of overall time budgets
#'
#' auxtime <- c(rep('morning', 8), rep('afternoon', 8), rep('night', 8))
#' simout <- simTimeBudget(dat, 1, aggregateTB = 'byvar', agVar  = auxtime, verbose = F) # ensemble of between-milking time budgets
#'
#' @export


simTimeBudget <- function(dat, nsim = 100,
                          simMethod = 'DUAL',
                          NTempSubsamp = NULL,
                          subsampVar = NULL,
                          aggregateTB = NULL,
                          agVar = NULL,
                          verbose = T){

  if(length(dim(dat)) == 2){ # simulate overall time budget

    matsim <- matrix(NA, nrow = nrow(dat), ncol = ncol(dat))
    rownames(matsim) <- rownames(dat)


    if(simMethod == 'DUAL'){
      matsim <- t(apply(dat, 1, function(x)
        rmultinom(1,
                  size = sum(x),
                  prob = DirichletReg::rdirichlet(1,
                                                  alpha = (x+1)
                  ))
      ))
      colnames(matsim) <- colnames(dat)
    }else if(simMethod == 'MN'){
      matsim <- t(apply(dat, 1, function(x)
        rmultinom(1,
                  size = sum(x),
                  prob = x/sum(x)
        )
      ))
      colnames(matsim) <- colnames(dat)
    }else if(simMethod == 'MVB'){
      matsim <- t(apply(dat, 1, function(x)
        DirichletReg::rdirichlet(1, alpha = (x+1) )
      ))
      colnames(matsim) <- colnames(dat)
    }else{
      stop('Error: simMethod option not recognized')
    }



    tbsim <- t(apply(matsim, 1, function(x) x/sum(x)))
    return(list(DataSim = matsim, TBSim = tbsim))


  }else{ #simulate time budget tensor

    tensorsim <- array(NA,
                       dim = c(dim(dat)[1],
                               dim(dat)[2],
                               dim(dat)[3],
                               nsim),
                       dimnames = list(unlist(dimnames(dat)[1]),
                                       unlist(dimnames(dat)[2]),
                                       unlist(dimnames(dat)[3]),
                                       1:nsim)
    )

    if(is.null(aggregateTB) | verbose){ # skip creating a second large matrix if we won't ever return it
      tbsim <- array(NA,
                     dim = c(dim(dat)[1],
                             dim(dat)[2],
                             dim(dat)[3],
                             nsim),
                     dimnames = list(unlist(dimnames(dat)[1]),
                                     unlist(dimnames(dat)[2]),
                                     unlist(dimnames(dat)[3]),
                                     1:nsim)  )
    }


    for(b in 1:nsim){

      ### subsample time interval ###
      if(is.null(NTempSubsamp)){

        myT <- unlist(dimnames(dat)[2])

      }else if(!is.null(NTempSubsamp) & is.null(subsampVar)){

        # subsample from time dimension unconditionally
        temp <- sample(unlist(dimnames(dat)[2]),
                       NTempSubsamp,
                       replace = F)
        myT <- unlist(dimnames(dat)[2])[unlist(dimnames(dat)[2])
                                        %in% temp]

      }else{

        if(length(subsampVar) != dim(dat)[2]){
          stop('Error: Dimension of subsample indexing variable must match length of temporal dimension')
        }

        temp <- sample(unique(subsampVar),
                       NTempSubsamp,
                       replace = F)
        myT <- unlist(dimnames(dat)[2])[subsampVar %in% temp]

      }


      ### resample obs data ###

      if(simMethod == 'DUAL'){

        #tensorsim <- tensorsim[,myT,,]
        tensorsim[,myT,,b] <- aperm(apply(dat[,myT,], c(1,2),
                                          function(x) rmultinom(1,
                                                                size = sum(x),
                                                                prob = DirichletReg::rdirichlet(1,
                                                                                                alpha = (x+1) ))),
                                    # convert to dierchlets mean
                                    perm = c(2,3,1))

      }else if(simMethod == 'MN'){

        #tensorsim <- tensorsim[,myT,,]
        tensorsim[,myT,,b] <- aperm(apply(dat[,myT,], c(1,2),
                                          function(x) rmultinom(1,
                                                                size = sum(x),
                                                                prob = x/sum(x) )),
                                    # convert to dierchlets mean
                                    perm = c(2,3,1))

      }else if(simMethod == 'MVB'){

        #tensorsim <- tensorsim[,myT,,]
        tensorsim[,myT,,b] <- aperm(apply(dat[,myT,], c(1,2),
                                          function(x)
                                            DirichletReg::rdirichlet(1, alpha = (x+1) )),
                                    perm = c(2,3,1))

      }else{
        stop('Error: simMethod option not recognized')
      }


      if(is.null(aggregateTB) | verbose){
        tbsim[,myT,,b] <- aperm(apply(tensorsim[,myT,,b], c(1,2),
                                      function(x) x/sum(x)),
                                perm = c(2,3,1))
      }

    }




    # View(dat[,1,,b])
    # View(tensorsim[,1,,b])
    # View(tbsim[,1,,b])
    # apply(tbsim[,1,,b], 1, sum)


    # for(i in 1:dim(tensorsim)[1]){
    #   for(j in 1:dim(tensorsim)[2]){
    #     for(b in 1:nsim){
    #
    #       temp <- dat[i,j, ]
    #       tensorsim[i,j, ,b] <- rmultinom(1,
    #                              size = sum(temp),
    #                              prob = DirichletReg::rdirichlet(1,
    #                                        alpha = (temp+1) )
    #                                      )
    #       tbsim[i,j, ,b] <- tensorsim[i,j, ,b]/sum(temp)
    #
    #
    #     }
    #   }
    # }



  }

  if(is.null(aggregateTB)){ # do not aggregate data by any time dimension

    return(list(DataSim = tensorsim,
                TBSim = tbsim,
                SimSpecs = list(simMethod = simMethod,
                                NTempSubsamp = NTempSubsamp,
                                subsampVar = subsampVar,
                                aggregateTB = aggregateTB,
                                agVar = agVar)))

  }else if(aggregateTB == 'overall'){

    tempc <- apply(tensorsim, c(1,3,4),
                   function(x) sum(x, na.rm = T))
    tempp <- aperm(apply(tempc, c(1,3),
                         function(x) x/sum(x, na.rm = T)),
                   perm = c(2,1,3))
    #View(Po)
    #View(tempp)

    if(verbose){
      return(list(DataSim = tempc,
                  TBSim = tempp,
                  FullSimResults = list(DataSim = tensorsim,
                                        TBSim = tbsim),
                  SimSpecs = list(simMethod = simMethod,
                                  NTempSubsamp = NTempSubsamp,
                                  subsampVar = subsampVar,
                                  aggregateTB = aggregateTB,
                                  agVar = agVar)
      ))
    }else{
      return(list(DataSim = tempc,
                  TBSim = tempp,
                  SimSpecs = list(simMethod = simMethod,
                                  NTempSubsamp = NTempSubsamp,
                                  subsampVar = subsampVar,
                                  aggregateTB = aggregateTB,
                                  agVar = agVar)))
    }


  }else if(aggregateTB == 'byvar'){

    if(is.null(agVar)){
      stop('Error: Provide vector with grouping variable for column indeces')
    }
    if(length(agVar) != dim(dat)[2]){
      stop('Error: AgVar vector does not match length of time dimension for data set provided')
    }

    tempc <- aperm(apply(tensorsim, c(1,3,4), function(x)
      tapply(x,agVar, sum)),
      perm = c(2,1,3,4))
    tempp <- aperm(apply(tempc, c(1,2,4), function(x) x/sum(x)),
                   perm = c(2,3,1,4))


    if(verbose){
      return(list(DataSim = tempc,
                  TBSim = tempp,
                  FullSimResults = list(DataSim = tensorsim,
                                        TBSim = tbsim),
                  SimSpecs = list(simMethod = simMethod,
                                  NTempSubsamp = NTempSubsamp,
                                  subsampVar = subsampVar,
                                  aggregateTB = aggregateTB,
                                  agVar = agVar)
      ))
    }else{
      return(list(DataSim = tempc,
                  TBSim = tempp,
                  SimSpecs = list(simMethod = simMethod,
                                  NTempSubsamp = NTempSubsamp,
                                  subsampVar = subsampVar,
                                  aggregateTB = aggregateTB,
                                  agVar = agVar)))
    }


  }


}
