#' Test for Significant Branching
#'
#' \code{testBranch} uses simulation-based techniques to determine if a branch represents an underlying biological signal that can be reliably differentiated from the intrinsic noise in the system
#'
#' @param obsdat Data matrix or array with observational units on the row axis. If data is generated through repeated sampling, time will be indexed on the second index, and features on the third and final axis. If data is not time indexed, features should be on the column axis
#' @param simdata Ensemble of data matrices simulated under the alternative hypothesis. Axes indices should match that observed data, with simulation index on the final axis.
#' @param rawdata Data array containing raw observations prior to any temporal aggregation.
#' @param feature_subset Optional argument. Default is to use all features in calculating dissimilarity values. User may pass a vector of feature names to calculate dissimilarity values for only a subset of features.
#' @param dist_method Optional argument. Dissimilarity estimate used to encode data. Options are 'Euclidean', 'KLD, or 'Ensemble'. Default is Euclidean.
#' @param weightdist Optional argument. Precomputed weight values that may be used to calculate ensemble weighted distances. See ensembleDist for more details
#' @param hclust_method Optional argument. Agglomeration method employed to create dendrogram. Default is 'ward.D2'. See hclust for more options.
#' @param nclustalt Optional integer argument. By default, if branch test with binary encoding is insignificant, branch will be pruned. User may elect to re-test branch with an alternative encoding granularity >2.
#' @param logbase Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).
#' @param SimSpecs Optional argument. Structured list containing all specifications needed to parameterize a simulated time budget. See simTimeBudget for formatting details
#' @param maxiter Optional argument. Maximum size that simulation under the alternative may be extended. Default is 2000. May be set to size of presimulated data set to prevent additional simulations from being performed to resolve ambiguous branch tests
#' @param B Optional arguement. Number of bootstrap simulations to perform under the null. Default is 2000.
#' @param showplot Optional logical argument. Default is FALSE. If TRUE density plot comparing MI values calculated against the alternative and null simulations is returned
#' @param verbose Optional logical argument. Default is FALSE. If TRUE additional simulation results are provided.
#'
#' @details Does a branch in a dendrogram represent reliable differences between two sets of observations driven by some underlying biological signal, or has a branch moved past the intrinsic resolution of the data set and thus only reflecting random meaningless fluctuations in measurements between observations? The brach test utility strives to answer this question using simulation-based appraches.
#'
#' Under the alternative, a branch under consideration contains an authentc signal that is only partially obscured by noise. Simulation-based techniques can be employed to create an ensemble of synthetic datasets mimick this noise structure. If the component of the observed data attributable to signal is significantly larger than this noise structure, then independently applying hclustering to each of these simulated data sets should produce a similar encoding to that extracted from the observed data.
#'
#' Under the null, any signal left in a branch is obscured by noise, and so the clustering is driven by stochastic flucturations. As this implies that all observations are equivalent in the branch up to the intrinsic resolution of the data, a second ensemble of mimicries is simulated under the null by resampling (bootsraping) values from the first mimicry unconditional on observation ID.
#'
#' Mutual Information is here used to evaluate how similar encodings of each simulated data set are to the encoding generated for the observed data. If a branch contains a signal that can be reliably distinguished from the simulated noise, the mutual information estimates to simulations under the alternative will be significantly higher than MI values evaluated against simulations under the null.
#'
#' If the 95th quantile of the null distribution falls below the 5th quantile under the alternative, the algorithm terminates, as a significant signal is detected. If an insignificant result is found, and if the user has specified an nclustalt option >2, the test will be repeated with an encoding with more clusters, which can help identify is two or more components of a complex signal with similar magnitudes are competing for branch position and artificially destabilizing the bifurcation. If a result is returned that places the two quantiles within the confidence bounds of the other, the test is repeated after doubling the simulation sizes to increase the overall power of the test. s
#'
#' @return A structured list containing:
#' \item{sigdiff}{Logical value indicating if the difference between simulated MI values under the null and the alternative exceeded the bounds of uncertainty of the simulations}
#' \item{pval}{Proportion of MI values under the alternative that were greater than or equal to the 95th quantile of MI values simulated under the null}
#' \item{encoding}{Vector containing the discrete encoding of the observed data}
#' \item{plot}{Density plot comparing MI values calculated against the alternative and null simulations}
#' \item{dist_method}{Returned if verbose is TRUE. Record of dissimilarity metric used to create hclust encoding}
#' \item{obsHTree}{Returned if verbose is TRUE. Dendrogram object returned from the heirarchical clustering of the observed data matrix}
#' \item{thresholdnull}{Returned if verbose is TRUE. Confidence interval for MI values calculated against the simulation under the null hypothesis}
#' \item{thresholdalt}{Returned if verbose is TRUE. Confidence interval for MI values calculated against the simulation under the alternative hypothesis}
#' \item{MI_null}{Returned if verbose is TRUE. Vector MI values calculated against the simulation under the null hypothesis}
#' \item{MI_alt}{Returned if verbose is TRUE. Vector of MI values calculated against the simulation under the alternative hypothesis}
#'
#'
#' @export




testBranch <- function(obsdat, simdata, rawdata, feature_subset = NULL, hclust_method = 'ward.D2', nclustalt = NULL, dist_method = 'Euclidean', weightdist = NULL, SimSpecs = NULL, logbase = 2, showplot = FALSE, verbose = F, B = 2000, maxiter = 2000){

  if(!is.null(feature_subset)){
    obsdat <- obsdat[ , feature_subset] # need to make dynamic
    simdata <- simdata[,feature_subset,]
    rawdata <- rawdata[ , , feature_subset]
    if(!is.null((weightdist))){
      weightdist <- weightdist[ , feature_subset]
    }
  }

  if(!is.null(nclustalt)){
    if(nrow(obsdat) < 2*nclustalt){nclustalt <- NULL}
    # if the user wants to look down the branch if the binary encoding is insignficant, but if there aren't at least twice as many obs as the number of encoding levels, then switch nclustalt to NULL so that we don't look down the branch with too few observations
  }



  ### get code for obs data ###

  encodeobs <- getEncoding(obsdat,
                             nclust = 2,
                             simdata = simdata,
                             hclust_method = hclust_method,
                             dist_method = dist_method,
                             weightdist = weightdist,
                             returnTree = T)

  # pheatmap(obsdat, cluster_rows = encodeobs$hctree) # check


  obscode <- encodeobs$cluster


  ###  Test for branch of n = 2 ###

  if(dim(obsdat)[1] == 2){ # if testing branch with only two obs
    return(list(sigdiff = testBranchTwo(obsdat, simdata),
                encoding = obscode))
  }

  ### get code for mimickry under alternative ###

  mimickcode_alt <- apply(simdata,
                          length(dim(simdata)),
                          function(x)
                            getEncoding(x,
                                          simdata = simdata,
                                          nclust = 2,
                                          hclust_method = hclust_method,
                                          dist_method = dist_method,
                                          #returnTree = T,
                                          weightdist = weightdist
                            ))

  # check dist max distance of alt trees matches obs tree
  # temp <- apply(simdata,
  #       length(dim(simdata)),
  #       function(x)
  #         max(getEncoding(x,
  #                       simdata = simdata,
  #                       nclust = 2,
  #                       hclust_method = hclust_method,
  #                       dist_method = dist_method,
  #                       returnTree = T,
  #                       weightdist = weightdist
  #         )$hctree$height )
  #       )
  # hist(temp)
  # check alt clust
  # temp <- getEncoding(as.matrix(simdata[,,5]),
  #                       simdata = simdata,
  #                       nclust = 2,
  #                       hclust_method = hclust_method,
  #                       dist_method = dist_method,
  #                       returnTree = T,
  #                       weightdist = weightdist)
  # plot(temp$hctree)
  # pheatmap(simdata[,,1], cluster_rows =  temp$hctree)


  ### make mimickry under null ###

  temp <- dimnames(obsdat)
  temp[[length(temp) + 1]] <- 1:B
  nullsim <- array(NA,
                   dim = c(dim(obsdat),B),
                   dimnames = temp)

  if(length(dim(obsdat))==2){ # simulate from overall time budget


    # fitdat <- rbind(as.matrix(obsdat),
    #                 matrix(aperm(simdata, c(3,1,2)),
    #                     ncol = dim(obsdat)[length(dim(obsdat))]))
    # myalpha <- paramMVB(fitdat) # MVB is fit to all observed and simulated otb vectors for this branch
    sampdat <- matrix(aperm(simdata, c(3,1,2)),
                      ncol = dim(obsdat)[length(dim(obsdat))])
    for(b in 1:B){
      #nullsim[,,b] <- DirichletReg::rdirichlet(nrow(obsdat),
      #                                 alpha = myalpha)
      nullsim[,,b] <- sampdat[sample(1:nrow(sampdat),
                                     dim(obsdat)[1]), ]
      # sample randomly from the full collection of simulated overall time vectors from any animal in the ensemble data set




    }

  }else{ # simulate from an obs data tensor with a time dimension

    # need to fix to also accept ensemble values
    # for(i in 1:dim(obsdat)[2]){
    #   myalpha <- paramMVB(obsdat[,i,])
    #   for(b in 1:B){
    #     nullsim[,i,,b] <- DirichletReg::rdirichlet(nrow(obsdat),
    #                                                alpha = myalpha)
    #   }
    # }

  }


  ### get code for null mimickry  ###

  if(length(dim(nullsim)) == 3){ # for analysis of overall TB matrix
    mimickcode_null <- apply(nullsim,
                             length(dim(nullsim)),
                             function(x)
                               getEncoding(x,
                                             nclust = 2,
                                             hclust_method = hclust_method,
                                             dist_method = dist_method,
                                             simdata = nullsim
                                             #weightdist = weightdist
                               )
    )
  }else{ # for analysis of tensor

  }



  ### get MI values ###

  mce_alt <- apply(mimickcode_alt, 2, function(x)
    calcEntropy(table(as.factor(obscode), as.factor(x)),
                logbase = logbase)$MutualInformation)

  mce_null <- apply(mimickcode_null, 2, function(x)
    calcEntropy(table(as.factor(obscode), as.factor(x)),
                logbase = logbase)$MutualInformation)

  nullquant <- EnvStats::eqnpar(mce_null, 0.95, ci = T,
                                lb = 0, ub = 1)
  nullthreshold <- nullquant$quantiles
  nullci <- nullquant$interval$limits

  if(length(unique(mce_alt)) == 1){ # if all sims have same cut
    altquant <- NULL
    altthreshold <- unique(mce_alt)
    altci <- c(unique(mce_alt), unique(mce_alt))
  }else{
    altquant <- EnvStats::eqnpar(mce_alt, 0.05, ci = T,
                                 lb = 0, ub = 1)
    altthreshold <- altquant$quantiles
    altci <- altquant$interval$limits
  }


  conftestfail <- sum(nullci >= altci[1] & nullci <= altci[2]) > 0 # check if quntile intervals overlap


  if(!conftestfail  &
     (nullthreshold < altthreshold |
      nullthreshold >= altthreshold & is.null(nclustalt))){
    # if you've got a confident test result then function should return results if either 1) the branch is sig for the binary case or 2) its not significant for the binary case but user doesn't want to look forward with multilevel encoding

    # plot test distributions
    plotdat = data.frame(MCE = c(mce_alt, mce_null),
                         Mimickry = c(rep('Alt',
                                          length(mce_alt)),
                                      rep('Null', length(mce_null)))
    )
    plotout <- ggplot(plotdat, aes(x = MCE, fill = Mimickry)) +
      geom_density(aes(y = ..scaled..), alpha = 0.75) +
      xlim(c(-0.1,1.1)) +
      ggtitle(paste('Result of Bifucation Simulation: N',
                    nrow(obsdat))) +
      ylab('Scaled Density') + xlab('Mutual Information') +
      scale_fill_manual(breaks = c("Alt", "Null"),
                        values=c("#00BFC4", "#F8766D"))
    if(showplot){
      print(plotout)
      # ggpubr::ggexport(plotout, width = 8, height = 5, res = 300,
      #    filename = 'Viz/OverallTB/EnsembleCut/BifucationViz.pdf')
    }

    # return output

    if(verbose){
      return(list(sigdiff = nullthreshold < altthreshold,
                  pval = sum(mce_alt <= nullthreshold) /
                    length(mce_alt),
                  encoding = obscode,
                  dist_method = dist_method,
                  obsHTree = encodeobs$hctree,
                  thresholdnull = nullquant,
                  thresholdalt = altquant,
                  MI_null = mce_null,
                  MI_alt = mce_alt,
                  plot = plotout))
    }else{
      return(list(sigdiff = nullthreshold < altthreshold,
                  pval = sum(mce_alt <= nullthreshold) /
                    length(mce_alt),
                  encoding = obscode,
                  plot = plotout
      ))
    }

  }else{ # either result is inconclusive, or negative and user wants to check down branch


    if(is.null(nclustalt)){

      # if user doesn't want to look forward, skip this step

      # check is multi-cluster necoding is significant
    }else{ # check is multi-cluster necoding is significant

      encodeobs2 <- getEncoding(obsdat,
                                  nclust = nclustalt,
                                  simdata = simdata,
                                  hclust_method = hclust_method,
                                  dist_method = dist_method,
                                  weightdist = weightdist,
                                  returnTree = T)
      obscode2 <- encodeobs2$cluster
      mimickcode_alt2 <- apply(simdata,
                               length(dim(simdata)),
                               function(x)
                                 getEncoding(x,
                                               simdata = simdata,
                                               nclust = nclustalt,
                                               hclust_method = hclust_method,
                                               dist_method = dist_method,
                                               weightdist = weightdist
                                 ))
      if(length(dim(nullsim)) == 3){ # analysis of overall TB matrix
        mimickcode_null2 <- apply(nullsim,
                                  length(dim(nullsim)),
                                  function(x)
                                    getEncoding(x,
                                                  simdata = nullsim,
                                                  nclust = nclustalt,
                                                  hclust_method = hclust_method,
                                                  dist_method = dist_method
                                    ))
      }else{ # for analysis of tensor

      }

      mce_alt2 <- apply(mimickcode_alt2, 2, function(x)
        calcEntropy(table(as.factor(obscode2), as.factor(x)),
                    logbase = logbase)$MutualInformation)
      mce_null2 <- apply(mimickcode_null2, 2, function(x)
        calcEntropy(table(as.factor(obscode2), as.factor(x)),
                    logbase = logbase)$MutualInformation)

      nullquant2 <- EnvStats::eqnpar(mce_null2, 0.95, ci = T)
      nullthreshold2 <- nullquant2$quantiles
      nullci2 <- nullquant2$interval$limits

      if(length(unique(mce_alt2)) == 1){ # if all sims have same cut
        altquant2 <- NULL
        altthreshold2 <- unique(mce_alt2)
        altci2 <- c(unique(mce_alt2), unique(mce_alt2))
      }else{
        altquant2 <- EnvStats::eqnpar(mce_alt2, 0.05, ci = T)
        altthreshold2 <- altquant2$quantiles
        altci2 <- altquant2$interval$limits
      }

      conftestfail2 <- sum(nullci2 >= altci2[1]
                           & nullci2 <= altci2[2]) > 0

      # return conditions

      if(!conftestfail2 ){
        # return if a confident result is returned, report reslts multi-cluster encoding
        # plot test distributions
        plotdat = data.frame(MCE = c(mce_alt2, mce_null2),
                             Mimickry = c(rep('Observed',
                                              length(mce_alt2)),
                                          rep('Null', length(mce_null2)))
        )
        plotout <- ggplot(plotdat, aes(x = MCE, fill = Mimickry)) +
          geom_density(aes(y = ..scaled..), alpha = 0.75) +
          ggtitle(paste('Result of Bifucation Simulation: N',
                        nrow(obsdat)))
        if(showplot){
          print(plotout)
        }

        # return output

        if(verbose){
          return(list(sigdiff = nullthreshold2 < altthreshold2,
                      pval = sum(mce_alt2 <= nullthreshold2) /
                        length(mce_alt2),
                      encoding = obscode,
                      dist_method = dist_method,
                      obsHTree = encodeobs2$hctree,
                      thresholdnull = nullquant2,
                      thresholdalt = altquant2,
                      MI_null = mce_null2,
                      MI_alt = mce_alt2,
                      plot = plotout))
        }else{
          return(list(sigdiff = nullthreshold2 < altthreshold2,
                      pval = sum(mce_alt2 <= nullthreshold2) /
                        length(mce_alt2),
                      encoding = obscode,
                      plot = plotout
          ))
        }

      }else if(nullthreshold >= altthreshold & !conftestfail & conftestfail2){
        # if the original test failed decisively, and now this test is inconclusive, return the original results
        # plot test distributions
        plotdat = data.frame(MCE = c(mce_alt, mce_null),
                             Mimickry = c(rep('Observed',
                                              length(mce_alt)),
                                          rep('Null', length(mce_null)))
        )
        plotout <- ggplot(plotdat, aes(x = MCE, fill = Mimickry)) +
          geom_density(aes(y = ..scaled..), alpha = 0.75) +
          xlim(c(-0.1,1.1)) +
          ggtitle(paste('Result of Bifucation Simulation: N',
                        nrow(obsdat)))
        if(showplot){
          print(plotout)
        }

        # return output

        if(verbose){
          return(list(sigdiff = nullthreshold < altthreshold,
                      pval = sum(mce_alt <= nullthreshold) /
                        length(mce_alt),
                      encoding = obscode,
                      dist_method = dist_method,
                      obsHTree = encodeobs$hctree,
                      thresholdnull = nullquant,
                      thresholdalt = altquant,
                      MI_null = mce_null,
                      MI_alt = mce_alt,
                      plot = plotout))
        }else{
          return(list(sigdiff = nullthreshold < altthreshold,
                      pval = sum(mce_alt <= nullthreshold) /
                        length(mce_alt),
                      encoding = obscode,
                      plot = plotout
          ))
        }
      }


    }

    # if we cont inconclusive results for binary and multi-clust case, then we must need to increase our sample size and try the test all over again


    altB <- dim(simdata)[length(dim(simdata))]
    if((2*altB) <= maxiter){
      if(length(dim(obsdat)) == 2){ # sim overal TB

        temp <- dim(simdata)
        temp[3] <- 2*altB
        temp2 <- dimnames(simdata)
        temp2[[3]] <- 1:temp[3]
        simdata2 <- array(NA, dim = temp, dimnames = temp2 )

        simdata2[ , , 1:altB] <- simdata
        simdata2[ , , (altB+1):(2*altB)] <- simTimeBudget(rawdata,
                                                          B = altB, # double size of sim
                                                          aggregateTB = 'overall',
                                                          simMethod = SimSpecs$simMethod,
                                                          NTempSubsamp = SimSpecs$NTempSubsamp,
                                                          subsampVar = SimSpecs$subsampVar,
                                                          agVar = SimSpecs$agVar,
                                                          verbose = F)$TBSim

        return(testBranch(obsdat, simdata2, rawdata,
                          SimSpecs = SimSpecs,
                          feature_subset = feature_subset,
                          hclust_method = hclust_method,
                          dist_method = dist_method,
                          weightdist = weightdist,
                          logbase = logbase ,
                          showplot = showplot,
                          verbose = verbose,
                          B = (B + 500),
                          maxiter = maxiter))
      }
    }else{
      # if we max out iterations, then just return a insignificant result from t he final binary test

      # plot test distributions
      plotdat = data.frame(MCE = c(mce_alt, mce_null),
                           Mimickry = c(rep('Observed',
                                            length(mce_alt)),
                                        rep('Null', length(mce_null)))
      )
      plotout <- ggplot(plotdat, aes(x = MCE, fill = Mimickry)) +
        geom_density(aes(y = ..scaled..), alpha = 0.75) +
        xlim(c(-0.1,1.1)) +
        ggtitle(paste('Result of Bifucation Simulation: N',
                      nrow(obsdat)))
      if(showplot){
        print(plotout)
      }

      # return output

      if(verbose){
        return(list(sigdiff = F,
                    pval = sum(mce_alt <= nullthreshold) /
                      length(mce_alt),
                    encoding = obscode,
                    dist_method = dist_method,
                    obsHTree = encodeobs$hctree,
                    thresholdnull = nullquant,
                    thresholdalt = altquant,
                    MI_null = mce_null,
                    MI_alt = mce_alt,
                    plot = plotout))
      }else{
        return(list(sigdiff = F,
                    pval = sum(mce_alt <= nullthreshold) /
                      length(mce_alt),
                    encoding = obscode,
                    plot = plotout
        ))
      }

    }


  }

  print('Missed a condition... dammit')

}


