#' Generate a tube plot
#'
#'  \code{tubeplot} creates a tube plot using the plotly package to visually display cyclical and longitudinal patterns in observed data.
#'
#' @param cyc A numeric vector containing cyclical indexing variable for each observation
#' @param long A numeric vector containing longitudinal temporal index for each observation
#' @param obs The observed value for each data point (ie - color of points). May be either continous or categorical
#' @param annotationkey Optional argument. A data frame mapping the numerical values given in c to text labels for cyclical time index. Defualts to no labels.
#' @param radius Optional argument. The radius of circle plotted for the cyclical axis. Default radius. Defualts to 1.
#' @param plot_title Optional argument. Title added to plot. Defaults to no title.
#' @param camera_orient Optional argument. Initial orientation of tube plot. Useful in defining correct orientation for generating still images. List with options for plotly camera settings. Defaults to list(eye = list(x = 1, y = 1, z = 0)
#' @param showplot Optional argument. TRUE/FALSE. Should plot be printed to screen. Default is TRUE
#' @param export_html Optional argument. TRUE/FALSE. Should interactive html file be saved to current working directory. Default is FALSE
#' @param filename Optional argument. Name of html file exported to current working directory. Do not include file extension. Default is 'TubePlot'
#' @param export_orca Optional argument. TRUE/FALSE. Should plot be exported to still jpeg image. Defaults to FALSE.
#' @param orcascale Optional argument. Size of orca jpeg image.
#' @param ... Additional plotting options passed to plotly. Used to control color scheme, marker formate, etc.
#'
#' @return Returns a 3D plot wherein observations appear as markers that are colored based on their observed value \code{obs} and arranged so that cyclical time indices \code{cyc} determine their arangement around the radius of the tube and longitudinal indices \code{long} determine their position along the shaft of the tube.
#' @examples
#' v_cyc <- rep(1:24, times = 40)
#' v_long <- rep(1:40, each = 24)
#' v_obs <- v_cyc + v_long
#' tubeplot(v_cyc, v_long, v_obs)
#'
#' @export



tubeplot <- function(cyc, long, obs, radius = 1, annotationkey = data.frame(), plot_title = '', camera_orient = list(eye = list(x = 1, y = 1, z = 0)),  showplot = T, export_html = F, filename = 'TubePlot', export_orca = F, orcascale = 10, ...){

  #require('plotly')

  h <- length(unique(cyc))
  d <- length(unique(long))

  # rescale h to have degrees of unit circle

  temp <- table(cyc)
  tempgap <- min(as.numeric(names(temp)[-1])-as.numeric(names(temp)[-length(temp)]))

  deg <- cyc + (0-min(cyc))
  deg <- (2*pi)/(max(cyc) + tempgap) * cyc

  # calculate coordinates on circle

  x <- radius*cos(deg)
  y <- radius*sin(deg)

  # set up data frame & annotations

  datplot <- data.frame(X = x, Y = y, Z = long, deg = deg, Obs = obs)

  temp <- subset(datplot, datplot$Z == min(datplot$Z))
  temp$H <- cyc[long == min(long)]
  temp <- temp[order(temp$H),]
  temp$Xnew <- (radius*1.2)*cos(temp$deg)
  temp$Ynew <- (radius*1.2)*sin(temp$deg)

  if(nrow(annotationkey)>0){
    names(annotationkey) <- c('H','text')
    temp <- merge(temp, annotationkey)
  }else{
    temp$text <- temp$H
  }

  myannotations <- list()

  for(i in seq(1, nrow(temp), by = 1)){
    myannotations[[length(myannotations)+1]] <- list(x = temp$Xnew[i],
                                                     y = temp$Ynew[i],
                                                     z = temp$Z[i],
                                                     text = as.character(temp$text[i]),
                                                     showarrow = F
                                                     #textangle = temp$deg[i] * 180/pi
    )
  }


  # plot this monster o.O

  tempa <- paste(getwd(), '/',filename ,'.html', sep = '')

  p1 <-  plotly::plot_ly(x=datplot$X,
                         y=datplot$Y,
                         z = datplot$Z,
                         color = datplot$Obs,
                         type="scatter3d",
                         mode="markers", ...) %>%
    plotly::layout(title = plot_title,
                   scene = list(annotations = myannotations,
                                camera = camera_orient)
                   )

  if(export_html){
    suppressWarnings(htmlwidgets::saveWidget(plotly::as_widget(p1), file = tempa, selfcontained = T))
  }


  if(export_orca){
    plotly::orca(p1, file = paste(filename, '.jpeg', sep = ''), scale = orcascale)
  }

  if(showplot){
    # ((x=datplot$X, y=datplot$Y, z = datplot$Z, color = datplot$Obs,
    #         type="scatter3d", mode="markers") %>%
    #   layout(title = plot_title, scene = list(annotations = myannotations,
    #                                           camera = camera_orient)
    #   )
    return(p1)
  }


}


