## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE-----------------------------------------------------
library(LIT)

library(dplyr)
library(ggplot2)
library(RColorBrewer)

## -----------------------------------------------------------------------------

library(labstats)
data(locomotor)

summary(locomotor)

locomotor[1:10,]


## ----fig.width = 4------------------------------------------------------------

library(lattice)

xyplot(dist ~ factor(time)|drug, data=locomotor, group=animal,
       type=c("g","l"), between=list(x=1), 
       col="black", lty=1, 
       xlab="Time (min)", ylab="Locomotor Activity",
       strip=strip.custom(bg="lightgrey"), 
       scales=list(alternating=FALSE) )


## -----------------------------------------------------------------------------

trtrats <- locomotor[locomotor$drug == 'Drug',]
locowide <- unstack(trtrats, dist~time)
rownames(locowide) <- paste('Rat',unique(trtrats$animal), sep = '')
names(locowide) <- c('15', '30','45','60','75','90')
locowide


## ----fig.width = 4, fig.height=6----------------------------------------------

dmout <- dmplot(locowide, n_rowclusters = 3, n_colclusters = 4,
                showplot_all = F, showplot_final = T,
                plot_title = 'Rat Data Mechanics Plot')


## ----fig.width = 4------------------------------------------------------------

# converting from wide to long using gather

library(tidyr)

locowide$DMClust <- dmout$rowclust
locowide$animal <- rownames(locowide)
locotrt <- gather(locowide, 
                   time, dist, '15':'90',
                   factor_key=F)
locotrt$time <- as.numeric(locotrt$time)
locotrt[1:10, ]

# plot treatment subgroups using lattice

library(lattice)

xyplot(dist ~ factor(time)|DMClust, 
       data=locotrt, group=animal,
       type=c("g","l"), between=list(x=1), 
       col="black", lty=1, 
       xlab="Time (min)", ylab="Locomotor Activity",
       strip=strip.custom(bg="lightgrey"), 
       scales=list(alternating=FALSE) )


## ---- include = FALSE---------------------------------------------------------

eodat <- read.csv('/Users/catie/Documents/Research/MyPackages/LITPackage/data/DMData.csv', header = T, row.names = 1)
substr(names(eodat), 4,4) <- ' '

dateaux <- read.csv('/Users/catie/Documents/Research/MyPackages/LITPackage/data/DMAuxData_col.csv', header = T, row.names = 1)

cowaux <- read.csv('/Users/catie/Documents/Research/MyPackages/LITPackage/data/DMAuxData_row.csv', header = T, row.names = 1)
rownames(cowaux) <- cowaux$CowID
cowaux <- cowaux[,c('Treatment', 'MilkYield95')]
  
  

## ----eval=FALSE---------------------------------------------------------------
#  
#  dmgrid_out <- dmplotGrid(eodat,
#                            n_rowclusters = 1:10,
#                            n_colclusters = 1:10,
#                            filename = 'Viz/EntryOrderPlot',
#                            imwidth = 10,
#                            imheight = 15,
#                            imres = 300)
#  

## ----fig.width=4, fig.height=6------------------------------------------------

dmgrid_out <- dmplotGrid(eodat, 
                          n_rowclusters = c(3,6), 
                          n_colclusters = c(2,3),
                          export_plots = F,
                          showplots = T)



## -----------------------------------------------------------------------------
str(dateaux)
names(eodat)[1:10]
dateaux[1:10,]

## -----------------------------------------------------------------------------
str(cowaux)
rownames('eodat')[1:10]
cowaux[1:10,]

## ----fig.width=4, fig.height=3------------------------------------------------

qplot(cowaux$MilkYield95, geom="histogram", xlab = 'Peak Milk Yeild', ylab = '', main = 'Distribution of Herd Productivity', col=I("black"), fill=I("blue"), alpha=I(.5), bins = 30) 


## ----fig.width=5, fig.height=4------------------------------------------------

# create data frame with Cow ID and continous milk yield variable
tempMY <- data.frame(CowID = rownames(cowaux), MilkYield95 = cowaux$MilkYield95) 
rownames(tempMY) <- rownames(cowaux) # add row names
tempMY <- tempMY[!is.na(tempMY$MilkYield95),] # remove missing values

# create hclust tree of milk yield values
hcout <- hclust(dist(tempMY$MilkYield95), method = 'ward.D2')
plot(hcout, xlab = 'Peak Milk Yeild')

# discretize data
nclust = 7
tempMY$MilkYield95C <- cutree(hcout, nclust)

#Create a intuitive category name using max and min milk yeild values for each cluster 
for(i in 1:nclust){
  
  temp2 <- subset(tempMY, tempMY$MilkYield95C == i)
  stringtemp <- paste(round(min(temp2$MilkYield95),1), 'lb-', 
                      round(max(temp2$MilkYield95),1), 'lb', sep = '')
  
  tempMY$MilkYield95C[tempMY$MilkYield95C == i] <- stringtemp
  
}

# order categorical variable

tempMY <- tempMY[order(tempMY$MilkYield95),]
tempMY$PeakYield <- factor(tempMY$MilkYield95C, 
                           levels =   unique(tempMY$MilkYield95C),
                           ordered = T)

# add categorical variable to cowaux
cowaux$CowID <- rownames(cowaux)
cowaux <- merge(cowaux, tempMY[,c('CowID','PeakYield')], by = 'CowID', all.x = T)
rownames(cowaux) <- cowaux$CowID
cowaux <- cowaux[, c('Treatment', 'PeakYield')]

cowaux[1:10, ]


## ----fig.width=6, fig.height=7------------------------------------------------

library(RColorBrewer)
col_list <- list(PeakYield = 
                setNames(
                  brewer.pal(nlevels(cowaux$PeakYield),"Reds"),
                  levels(cowaux$PeakYield)),
                Treatment  = c('Control' = 'cornflowerblue',
                               'Organilac' = 'gold'),
                Period = c('Pen' = 'cornflowerblue', 
                           'Pasture' = 'darkolivegreen1'),
                DaysOnTrial  = brewer.pal(dateaux$DaysOnTrial, "Purples")
                       )

dmout <- dmplot(eodat,
                n_rowclusters = 7,
                n_colclusters = 4,
                export_plot_final = F,
                plot_title = 'Entry Order DM Plot',
                annotation_row = cowaux, 
                annotation_col = dateaux,
                annotation_colors = col_list)


