% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivarTreeTest.R
\name{bivarTreeTest}
\alias{bivarTreeTest}
\title{Mutual Information Test for Tree Structures}
\usage{
bivarTreeTest(
  inputdatx,
  inputdaty,
  grid_x = 2:5,
  grid_y = 2:5,
  logbase = 2,
  nperm = 1000,
  estimator = "ML",
  rescaleMI = T,
  showplot = T
)
}
\arguments{
\item{inputdatx}{First encoded variable. User may pass either a list output from other LIT clustering utilities, a dendrogram, or a data frame formatted with named rows and one column containing the encoded variable.}

\item{inputdaty}{Second encoded variable. User may pass either a list output from other LIT clustering utilities, a dendrogram, or a data frame formatted with named rows and one column containing the encoded variable.}

\item{grid_x}{Vectors of integer values containing the range of potential cluster numbers with which to encode variable x.}

\item{grid_y}{Vectors of integer values containing the range of potential cluster numbers with which to encode variable y.}

\item{logbase}{Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).}

\item{nperm}{Optional argument. Defualt is 1000. Number of permutations used to simulate the null MI value for each pairwise comparision of candidate tree encodings}

\item{estimator}{Optional argument. Method employed to estimate empirical frequencies of the joint encoding used to calculate entropic values. Defualt is 'ML'. Bias-corrected estimators available include "shrink", "Jeffreys","Laplace", "SG", "minmax".}

\item{rescaleMI}{Optional logical argument. If TRUE (default), estimator used in permutation tests for signicance of association is MI rescaled by the total entropy of the joint encoding. If FALSE, the raw/unscaled MI estimate is used as the standard of comparison.}

\item{showplot}{Optional logical argument. Default is TRUE, which returns a heatmap visualization of the best MI test results for each metaprameter combination across the grid of cluster number values.}
}
\value{
A structured list containing:
\item{PVals}{A multidimensional array, with indices matching the clustering indices, containing the p-value results for the corresponding MI test}
\item{MIVals}{A multidimensional array, with indices matching the clustering indices, containing the observed MI values of the encoded data}
\item{ZScores}{A multidimensional array,  with indices matching the clustering indices, containing the observed MI values of the encoded data centered and scaled by the median and inter-quartile range of the MI values simulated under the null}
\item{PlotOut}{NULL if showplot is FALSE. A pheatmap object containing a visualization of the best simulated MI Z-scores across the metaparmeter grid of clustering granularities.}
}
\description{
\code{bivarTreeTest} allows users to test for a significant bivariate relationship between two discretized variables using an MI test on a grid of encoding granularities (encoding metaparameter).
}
\details{
When working with complex data sets that are empirically discretized using clustering techniques, there is typically no obvious or intrisic granularity in the subseqent encoding. Subsequently, it can be challenging to determine a priori the optimal combination of encoding granularities to test for a significant bivariate relationship between encodings.

The bivarTreeTest utility allows users to conduct mutual information tests on a grid of encoding granularity values, and provides visualization of simulation results to aid in selecting the optimal metaparameter values where bivariate relationship come into optimal focus. This ultimately provides a more comprehsive assessment of the relationships between tree structures.

Users may input either LIT clustering output, raw dendrograms, or a discrete encoding provided as a dataframe. If the user passes output from a data mechanics clustering (dmplot), multiple MI tests will be conducted at each encoding granularity gridpoint, with full results returned in SimResults, and the best test result used in the optimization visualization. This allows users to not only explore the best encoding granularity between observational units, but even optimize for temporal/multivariate structures across experimental units.
}
\examples{

library(labstats)
data(locomotor)

locowide <- unstack(locomotor, dist~time)
rownames(locowide) <- paste('Rat', unique(locomotor$animal), sep = '')
names(locowide) <- c('Min15', 'Min30','Min45','Min60','Min75','Min90')
auxrow <- unique(locomotor[, c('animal', 'drug')])
rownames(auxrow) <- paste('Rat', auxrow$animal, sep = '')
auxrow <- auxrow[,'drug', drop = F]
names(auxrow) <- 'Treatment'

encodeout <- encodePlot(locowide, dist(locowide), dist(t(locowide)),
                    n_rowclusters = 2:5, n_colclusters = 0,
                    plot_title = 'Rat Locomotion Pattern Encoding',
                    annotation_row = auxrow)

myx <- as.data.frame(x = round(runif(dim(locowide)[1], 0, 5)))
rownames(myx) <- rownames(auxrow)

bvout <- bivarTreeTest(encodeout, myx, grid_x = 2:5, grid_y = 5)
bvout$PVals

}
