% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcEntropy.R
\name{calcEntropy}
\alias{calcEntropy}
\title{Calculate Entropic Measures}
\usage{
calcEntropy(
  counts,
  logbase = 2,
  estimator = "ML",
  isfreq = F,
  verbose = F,
  ...
)
}
\arguments{
\item{counts}{Vector, matrix, or multidimensional array containing count data, wherein each axis represents a discete encoding.}

\item{logbase}{Optional argument. Logarithm base to use in computing entropic values. Default is 2 (Shannon's Entropy).}

\item{estimator}{Optional argument. Estimator used to calculate empirical fequencies. Defualt is 'ML'. Bias-corrected estimators available include "shrink", "Jeffreys","Laplace", "SG", "minmax".}

\item{isfreq}{Optional arguement. Default is FALSE, which assumes input is a count matrix. When TRUE you may directly pass a freqeuncy matrix into entropy claculations, with no normalization or bias correction.}

\item{verbose}{Optional arguement. Default is FALSE. If TRUE, output will contain frequency estimates used to calculate entropic values.}

\item{...}{Additional options passed to entropy() calculated using the entropy package is an estimator of type not 'ML' is specified.}
}
\value{
Returns a list containing:

For univariate data:
\item{Entropy}{Estimate of stochasiticity of observed ensemble}
\item{MaxEntropy}{Estimate of the maximum theoretical stochasticity that could be acheived in an observed system if all encodings were equi-probable}
\item{Redundancy}{Comparison of the stochasiticty of an observed ensemble to the maximum theoretically-achievable entropy of that system}

For bivariate data:
\item{MarginalEntropyRow}{List containing univariate entropy values of observations for the row variable encoding aggregated across the column margin}
\item{MarginalEntropyCol}{List containing univariate entropy values of observations for the column variable encoding aggregated across the row margin}
\item{JointEntropy}{Entropy of the joint encoding}
\item{ConditionEntropyRow}{Conditional entropy remaining in the row variable after accounting for the information contained in the column variable encoding}
\item{ConditionEntropyCol}{Conditional entropy remaining in the column variable after accounting for the information contained in the row variable encoding}
\item{MutualInformation}{Mutual information shared between encodings of the row and column variables}
\item{RelativeMutualInformation}{Mutual information rescaled by the joint entropy. Proportion of total joint entropy shared between the two encodings}

For trivariate data:

\item{JointEntropy}{Entropy of the joint encoding}
\item{MutualInformation}{Mutual information shared between encodings of the row and column variables. Positive values indicate informational redundancy, while negative values may indicate synergy between variables (see James & Crutchfield 2016). It should be noted that in sparsely sampled datasets, empirical frequency estimates tend to produce negative bias in three-way mutual information estimates}
\item{MCE_XY}{Mutual information shared between encodings X and Y but not with encoding Z}
\item{MCE_XZ}{Mutual information shared between encodings X and Z but not with encoding Y}
\item{MCE_YZ}{Mutual information shared between encodings Y and Z but not with encoding X}
\item{MarginalMI_XY}{List containing the marginal bivariate entropic estimates for encodings X and Y independent of encoding Z}
\item{MarginalMI_XZ}{List containing the marginal bivariate entropic estimates for encodings X and Z independent of encoding Y}
\item{MarginalMI_YZ}{List containing the marginal bivariate entropic estimates for encodings Y and Z independent of encoding X}
\item{TotalCorrelation}{Sum of three-way mutual information and all three mutual conditional entropy estimates}
\item{MI_TotCor}{Three-way mutual information scaled/normalized by the total correlation estimate}
}
\description{
\code{calcEntropy} computes an array of information theoretic measures to characterize univariate, bivariate, or multivariate systems of discrete encodings
}
\details{
calcEntropy returns a range of useful information theoretic estimates for univ-, bi-, and tri-variate encodings.

By default, entorpy is calulated using the Maximum Likelihood Estimator. Where the number of observations in the dataset is not considerably larger than the number of categories that a discrete variable is divided into (N >> k), univariate entropy estimates will be negatively biased, which will in turn bias all higher-order mutual information estimates. This will be especially true for use cases where the marginal or joint probability distributions contain many low-probability bins with observation counts of zero. Where this is an issue, the user may choose from several bias-corrected empirical frequency estimation techniques computed using the Entropy package (Hausser & Strimmer 2021).

Formulae for information theoretic estimates implemented in this function follow conventions from Mackay (2003)
}
\examples{

# Example 1: Univariate data

obsdata <- c('A', 'B', 'C', 'C') \% observed data from system that can take encoded values A-D

mycounts <- table(obsdata)
mycounts
calcEntropy(mycounts)

mycounts <- table(factor(obsdata, levels = c('A', 'B', 'C', 'D')))
mycounts
calcEntropy(mycounts) # note that entropy estimate does not change, but max entropy and subsequently redundancy estimates will, with inclusion of empty bins

# Example 2: Bivariate Data

jointcount <- matrix(0, nrow = 4, ncol = 4)
jointcount[1,1] <- 1
jointcount[1,2] <- 1
jointcount[2,3] <- 1
jointcount[2,4] <- 1
jointcount[3,1] <- 1
jointcount[3,3] <- 1
jointcount[4,2] <- 1
jointcount[4,4] <- 1
jointcount

calcEntropy(jointcount)

# Example 3: Trivariate Data (from James & Crutchfield 2016)

X <- c(0,1,0,1,2,3,2,3)
Y <- c(0,1,2,3,0,1,2,3)
Z <- c(0,1,2,3,2,3,0,1)

counts <- table(X,Y,Z)
calcEntropy(counts)


}
